
#include <iostream>
#include <getopt.h>

#include <QApplication>
#include <QTranslator>
#include <QLibraryInfo>
#include <QString>

#include "snd_control_info.hpp"
#include "mixer_main_window.hpp"
#include "config.hpp"
#include "init_globals.hpp"


///
/// @brief Command line option parsing
///

int
parse_options (
	int argc,
	char * argv[],
	QString & dev_str_n );


///
/// @brief The main function
///

int
main (
	int argc,
	char * argv[] )
{
	int res ( 0 );

	init_globals();

	QString arg_dev_str;

	res = parse_options ( argc, argv, arg_dev_str );
	if ( res != 0 ) {
		return res;
	}


	// Setup the QApplication
	QApplication app ( argc, argv );

	// Setup the translator

	QTranslator translator_qt;
	QTranslator translator_local;

	translator_qt.load ( "qt_" + QLocale::system().name(),
            QLibraryInfo::location ( QLibraryInfo::TranslationsPath ) );

	translator_local.load ( "app_" + QLocale::system().name(),
		INSTALL_DIR_L10N );

	app.installTranslator ( &translator_qt );
	app.installTranslator ( &translator_local );


	{
		// Setup the application icon
		QString icon_path ( INSTALL_DIR_ICONS_PNG_64 );
		icon_path += "/";
		icon_path += PROGRAM_NAME;
		icon_path += ".png";
		app.setWindowIcon ( QIcon ( icon_path ) );
	}


	// Mixer window

	Mixer_Main_Window mixer_win;
	mixer_win.hide();

	// Select control device from the command line o the default one
	if ( !arg_dev_str.isEmpty() ) {
		QSnd::Snd_Control_Info ctl_info ( arg_dev_str );
		mixer_win.set_current_ctl ( ctl_info );
	} else {
		mixer_win.set_default_ctl();
	}

	// Show mixer
	mixer_win.resize ( 800, 450 );
	mixer_win.show();

	res = app.exec();

	return res;
}



const char copy_txt[] =
"\
alsamixer-qt4 - A sound volume mixer application\n\
\n\
Copyright (C) 2010  Sebastian Holtermann\n\
\n\
This program is free software: you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation, either version 3 of the License, or\n\
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program.  If not, see <http://www.gnu.org/licenses/>.\n\
";


int
parse_options (
	int argc,
	char * argv[],
	QString & dev_str_n )
{
	// Suppresses error messages by getopt_long
	opterr = 0;

	QString card_idx_str;
	QString device_str;

	bool flag_print_version ( false );
	bool flag_print_help ( false );
	bool flag_print_copy_info ( false );

	bool scan_further ( true );

	while ( scan_further ) {

		static struct option long_opts[] =
		{
			{ "card",    required_argument, 0, 'c' },
			{ "device",  required_argument, 0, 'D' },
			{ "help",    no_argument,       0, 'h' },
			{ "copying", no_argument,       0, 'i' },
			{ "version", no_argument,       0, 'v' },
			{0, 0, 0, 0}
		};

		// getopt_long stores the option index here.
		int long_opts_idx ( 0 );
		int opt_char = getopt_long ( argc, argv, "hc:D:iv",
			long_opts, &long_opts_idx );

		// Leave loop
		if ( opt_char < 0 ) {
			break;
		}

		switch ( opt_char ) {
			case 0:
				break;

			case 'c':
				// Card index
				card_idx_str = optarg;
				break;

			case 'D':
				// Device string index
				device_str = optarg;
				break;

			case 'v':
				flag_print_version = true;
				break;

			case 'h':
				flag_print_help = true;
				scan_further = false;
				break;

			case 'i':
				flag_print_copy_info = true;
				break;

			default:
				// Dont't break, as the option may be for QT
				break;
		}
	}

	if ( flag_print_help ) {
		std::cout << "Usage: " << PROGRAM_NAME << " [options]\n";
		std::cout << "Options:\n";
		std::cout << "  -h, --help          prints this help\n";
		std::cout << "  -c, --card=NUMBER   selects a card\n";
		std::cout << "  -D, --device=NAME   selects a mixer device (e.g. hw:1)\n";
		std::cout << "  -i, --copying       prints copying informations\n";
		std::cout << "  -v, --version       prints the program version\n";
		return -1;
	}

	if ( flag_print_version ) {
		std::cout << PROGRAM_NAME << " " << VERSION << std::endl;
		return -1;
	}

	if ( flag_print_copy_info ) {
		std::cout << copy_txt;
		return -1;
	}

	if ( !card_idx_str.isEmpty() ) {
		if ( device_str.isEmpty() ) {
			bool is_numeric;
			const int card_idx ( card_idx_str.toInt ( &is_numeric ) );
			if ( is_numeric ) {
				device_str = QString ( "hw:%1" ).arg ( card_idx );
			}
		}
	}

	if ( !device_str.isEmpty() ) {
		dev_str_n = device_str;
	}

	return 0;
}
