//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "mixer_main_window.hpp"

#include <QVBoxLayout>
#include <QCoreApplication>

#include <iostream>

//#include "qslider_test_dialog.hpp"


Mixer_Main_Window::Mixer_Main_Window (
	QWidget * parent,
	Qt::WindowFlags flags ) :
QMainWindow ( parent, flags ),
_qsnd_mixer ( this ),
_dialg_info ( 0 )
{
	setWindowTitle ( PROGRAM_TITLE );

	{
		int mg ( 8 );
		setContentsMargins ( mg, mg, mg, mg );
	}

	_mixer_config.show_dir[0] = true;
	_mixer_config.show_dir[1] = false;

	mixer_sliders().set_pixmaps_hub ( &_pixmaps_hub );
	mixer_sliders().set_mixer_config ( _mixer_config );
	mixer_switches().set_mixer_config ( _mixer_config );


	// Side interface

	_side_iface.setFrameStyle ( QFrame::StyledPanel | QFrame::Plain );
	_side_iface.set_mixer_config ( _mixer_config );

	connect ( &_side_iface, SIGNAL ( sig_control_changed() ),
		this, SLOT ( update_current_ctl() ) );

	connect ( &_side_iface, SIGNAL ( sig_mixer_config_changed() ),
		this, SLOT ( update_mixer_config() ) );


	// Init menu
	init_menu();


	// Snd_Mixer_Simple

	connect ( &_qsnd_mixer, SIGNAL ( sig_mixer_elements_changed() ),
		this, SLOT ( mixer_elements_changed() ) );


	// Horizontal splitter

	int hwidth ( 10 + (_split_vert.handleWidth() % 2 ) );

	// Compose central layout
	_split_hor.setOrientation ( Qt::Horizontal );
	_split_hor.setHandleWidth ( hwidth );
	_split_hor.addWidget ( &_mixer_sliders );
	_split_hor.addWidget ( &_side_iface );
	_split_hor.setCollapsible ( 0, false );
	_split_hor.setCollapsible ( 1, false );
	_split_hor.setStretchFactor ( 0, 1 );
	_split_hor.setStretchFactor ( 1, 0 );


	// Vertical splitter

	_split_vert.setOrientation ( Qt::Vertical );
	_split_vert.setHandleWidth ( hwidth );
	_split_vert.addWidget ( &_split_hor );
	_split_vert.addWidget ( &_mixer_switches_area );
	_split_vert.setCollapsible ( 0, false );
	_split_vert.setCollapsible ( 1, true );


	setCentralWidget ( &_split_vert );

	refresh_data();

	// Slider test dialog
	//Wdg::QSlider_Test_Dialog * sl_test = new Wdg::QSlider_Test_Dialog ( this );
	//sl_test->show();
}


Mixer_Main_Window::~Mixer_Main_Window ( )
{
	mixer_sliders().set_mixer_simple ( 0 );
	mixer_switches().set_mixer_simple ( 0 );
	_qsnd_mixer.close_mixer();
}


void
Mixer_Main_Window::init_menu ( )
{
	// Action: Exit / Quit
	_act_exit = new QAction ( tr ( "&Quit" ), this );
	_act_exit->setShortcut ( QKeySequence ( QKeySequence::Quit ) );

	// Action: Refresh
	_act_refresh = new QAction ( tr ( "&Refresh" ), this );
	_act_refresh->setShortcut ( QKeySequence ( QKeySequence::Refresh ) );

	// Action: Fullscreen
	_act_fullscreen = new QAction ( tr ( "&Fullscreen" ), this );
	_act_fullscreen->setShortcut ( QKeySequence ( tr ( "Ctrl+F" ) ) );
	_act_fullscreen->setCheckable ( true );

	// Action: Info
	_act_info = new QAction ( tr ( "&Info" ), this );
	_act_info->setShortcut ( QKeySequence ( QKeySequence::HelpContents ) );


	// Menus

	_menu_file = menuBar()->addMenu ( tr ( "&File" ) );
	_menu_file->addAction ( _act_exit );

	_menu_view = menuBar()->addMenu ( tr ( "&View" ) );
	_menu_view->addAction ( _act_refresh );
	_menu_view->addAction ( _act_fullscreen );

	_menu_help = menuBar()->addMenu ( tr ( "&Help" ) );
	_menu_help->addAction ( _act_info );


	// Connect

    connect ( _act_exit, SIGNAL ( triggered() ),
		this, SLOT ( close() ) );

    connect ( _act_refresh, SIGNAL ( triggered() ),
		this, SLOT ( refresh_data() ) );

    connect ( _act_fullscreen, SIGNAL ( triggered() ),
		this, SLOT ( toggle_fullscreen() ) );

    connect ( _act_info, SIGNAL ( triggered() ),
		this, SLOT ( show_dialog_info() ) );
}


void
Mixer_Main_Window::set_current_ctl (
	const QSnd::Snd_Control_Info & ctl_n )
{
	//std::cout << "set_current_ctl " << ctl_n.ctl_id().toLocal8Bit().data() << "\n";

	if ( _current_ctl != ctl_n ) {
		_current_ctl = ctl_n;

		// Clear mixer
		if ( _qsnd_mixer.is_open() ) {
			mixer_sliders().set_mixer_simple ( 0 );
			mixer_switches().set_mixer_simple ( 0 );
			_qsnd_mixer.close_mixer();
		}

		if ( !_current_ctl.is_clear() ) {
			_qsnd_mixer.open_mixer ( _current_ctl.ctl_id() );
		}

		if ( _qsnd_mixer.is_open() ) {
			mixer_switches().set_mixer_simple ( &_qsnd_mixer );
			mixer_sliders().set_mixer_simple ( &_qsnd_mixer );
			reset_vsplitter();
		}

		_side_iface.set_current_ctl ( _current_ctl );
	}
}


void
Mixer_Main_Window::set_default_ctl ( )
{
	set_current_ctl ( _side_iface.default_ctl() );
}


void
Mixer_Main_Window::update_current_ctl ( )
{
	set_current_ctl ( _side_iface.current_ctl() );
}


void
Mixer_Main_Window::set_mixer_config (
	const QSnd::Mixer_Config & cfg_n )
{
	if ( _mixer_config != cfg_n ) {
		_mixer_config = cfg_n;

		_side_iface.set_mixer_config ( cfg_n );

		mixer_switches().set_mixer_config ( _mixer_config );
		mixer_sliders().set_mixer_config ( _mixer_config );

		reset_vsplitter();
	}
}


void
Mixer_Main_Window::update_mixer_config ( )
{
	set_mixer_config ( _side_iface.mixer_config() );
}


bool
Mixer_Main_Window::event (
	QEvent * event_n )
{
	bool res ( true );

	if ( event_n->type() == QSnd::event_type_refresh_data ) {
		refresh_data();
	} else {
		res = QMainWindow::event ( event_n );
	}

	return res;
}


void
Mixer_Main_Window::closeEvent (
	QCloseEvent * event )
{
	if ( _dialg_info != 0 ) {
		_dialg_info->close();
	}
	QMainWindow::closeEvent ( event );
}


void
Mixer_Main_Window::reset_hsplitter ( )
{
	const int ww_tot ( _split_hor.width() - _split_hor.handleWidth() );
	const int ww_sb ( _side_iface.sizeHint().width() );
	QList < int > sizes;
	sizes << ww_tot - ww_sb;
	sizes << ww_sb;

	_split_hor.setSizes ( sizes );
}


void
Mixer_Main_Window::reset_vsplitter ( )
{
	const int hh ( ( _split_vert.height() - _split_vert.handleWidth() ) / 2 );
	QList < int > sizes;
	sizes << hh;
	sizes << hh;
	_split_vert.setSizes ( sizes );
}


void
Mixer_Main_Window::show_dialog_info ( )
{
	if ( _dialg_info == 0 ) {
		_dialg_info = new Dialog_Info ( );
		_dialg_info->setAttribute ( Qt::WA_DeleteOnClose );
	}
	_dialg_info->show();
}


void
Mixer_Main_Window::mixer_elements_changed ( )
{
	//std::cout << "Mixer_Main_Window::mixer_elements_changed" << "\n";
	QCoreApplication::postEvent (
		this, new QEvent ( QSnd::event_type_refresh_data ) );
}


void
Mixer_Main_Window::refresh_data ( )
{
	QSnd::Snd_Control_Info ctl_store ( _current_ctl );
	set_current_ctl ( QSnd::Snd_Control_Info() );

	reset_hsplitter();
	_side_iface.refresh_data();

	set_current_ctl ( ctl_store );
}


void
Mixer_Main_Window::toggle_fullscreen ( )
{
	const bool is_fs ( windowState() & Qt::WindowFullScreen );
	if ( is_fs ) {
		showNormal();
	} else {
		showFullScreen();
	}
	_act_fullscreen->setChecked ( !is_fs );
}

