//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "mixer_sliders.hpp"

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>

#include <QCoreApplication>
#include <QKeyEvent>


namespace QSnd
{


//
// Mixer_Sliders
//
Mixer_Sliders::Mixer_Sliders (
	QWidget * parent_n ) :
QWidget ( parent_n ),
_pixmaps_hub ( 0 ),
_sliders_scroll_pad ( this ),
_status_bar ( _sliders_scroll_pad.sliders_pad() ),
_context_menu ( this ),
_act_separate_join ( this ),
_act_level_volumes ( this ),
_act_separator_channels ( this ),
_act_mute_all ( this ),
_act_unmute_all ( this ),
_mixer_simple ( 0 )
{
	_sliders_scroll_pad.setFrameStyle ( QFrame::StyledPanel | QFrame::Plain );
	_status_bar_frame.setFrameStyle ( QFrame::StyledPanel | QFrame::Plain );

	sliders_pad()->installEventFilter ( this );

	// Actions
	_act_separate_join.setShortcut ( QKeySequence ( tr ( "c" ) ) );
	_act_level_volumes.setShortcut ( QKeySequence ( tr ( "l" ) ) );
	_act_mute_all.setShortcut ( QKeySequence ( tr ( "m" ) ) );
	_act_unmute_all.setShortcut ( QKeySequence ( tr ( "u" ) ) );

	_act_separator_channels.setSeparator ( true );

	_act_level_volumes.setText ( tr ( "&Level volumes" ) );

	_act_str_toggle_separate[0] = tr ( "&Split channels" );
	_act_str_toggle_separate[1] = tr ( "&Join channels" );
	_act_str_mute[0] = tr ( "&Mute" );
	_act_str_mute[1] = tr ( "&Mute all" );
	_act_str_unmute[0] = tr ( "&Unmute" );
	_act_str_unmute[1] = tr ( "&Unmute all" );

	connect ( &_act_separate_join, SIGNAL ( triggered ( bool ) ),
		this, SLOT ( action_toggle_joined_separated() ) );

	connect ( &_act_level_volumes, SIGNAL ( triggered ( bool ) ),
		this, SLOT ( action_level_volumes() ) );

	connect ( &_act_mute_all, SIGNAL ( triggered ( bool ) ),
		this, SLOT ( action_mute_all() ) );

	connect ( &_act_unmute_all, SIGNAL ( triggered ( bool ) ),
		this, SLOT ( action_unmute_all() ) );


	// Context menu
	_context_menu.addAction ( &_act_separate_join );
	_context_menu.addAction ( &_act_level_volumes );
	_context_menu.addAction ( &_act_separator_channels );
	_context_menu.addAction ( &_act_unmute_all );
	_context_menu.addAction ( &_act_mute_all );


	// Separation timer
	_timer_separation.setSingleShot ( false );
	_timer_separation.setInterval ( 200 );
	connect ( &_timer_separation, SIGNAL ( timeout() ),
		this, SLOT ( timed_separation_check() ) );


	// Layout
	QVBoxLayout * lay_status ( new QVBoxLayout() );
	lay_status->setContentsMargins ( 4, 4, 4, 4 );
	lay_status->addWidget ( &_status_bar );
	_status_bar_frame.setLayout ( lay_status );

	QVBoxLayout * lay_vbox ( new QVBoxLayout() );
	lay_vbox->setContentsMargins ( 0, 0, 0, 0 );
	lay_vbox->setSpacing ( 6 );
	lay_vbox->addWidget ( &_sliders_scroll_pad, 1 );
	lay_vbox->addWidget ( &_status_bar_frame, 0 );

	setLayout ( lay_vbox );
}


Mixer_Sliders::~Mixer_Sliders ( )
{
	set_mixer_simple ( 0 );
}


void
Mixer_Sliders::set_mixer_simple (
	Snd_Mixer_Simple * mixer_n  )
{
	if ( _mixer_simple != 0 ) {
		disconnect ( _mixer_simple, 0, this, 0 );
		disconnect ( _mixer_simple, 0, &_status_bar, 0 );
	}

	sliders_pad()->set_proxies_groups ( 0 );
	clear_proxies_groups();

	_mixer_simple = mixer_n;

	if ( _mixer_simple != 0 ) {

		connect ( _mixer_simple, SIGNAL ( sig_mixer_values_changed() ),
			this, SLOT ( mixer_values_changed() ) );

		connect ( _mixer_simple, SIGNAL ( sig_mixer_values_changed() ),
			&_status_bar, SLOT ( mixer_values_changed() ) );

		create_proxies_groups();
		update_layout();
	}

}


void
Mixer_Sliders::set_mixer_config (
	const Mixer_Config & cfg_n  )
{
	if ( !_config.show_dirs_equal ( cfg_n ) ) {
		_config = cfg_n;
		update_layout();
	}
}


void
Mixer_Sliders::set_pixmaps_hub (
	Pixmaps_Hub * hub_n )
{
	_pixmaps_hub = hub_n;

	Wdg::QSlider_Pixmaps_Buffers sl_buff;
	Wdg::QCheck_Box_Pixmaps_Buffers cb_buff;

	if ( _pixmaps_hub != 0 ) {
		sl_buff = _pixmaps_hub->slider_buffers();
		cb_buff = _pixmaps_hub->check_box_buffers();
	}

	sliders_pad()->set_pixmaps_buffers ( sl_buff, cb_buff );
}


void
Mixer_Sliders::clear_proxies_groups ( )
{
	for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
		delete _proxies_groups[ii];
	}
	_proxies_groups.clear();
	_proxies_groups_visible.clear();
}


void
Mixer_Sliders::create_proxies_groups ( )
{
	if ( _mixer_simple == 0 ) {
		return;
	}

	for ( int ii=0; ii < _mixer_simple->elems().size(); ++ii ) {
		Snd_Mixer_Simple_Elem * qsme ( _mixer_simple->elems()[ii] );

		for ( int snd_dir = 0; snd_dir < 2; ++snd_dir ) {
			if ( qsme->has_volume ( snd_dir ) )
			{
				Mixer_Sliders_Proxies_Group * mspg (
					new Mixer_Sliders_Proxies_Group ( this ) );
				mspg->set_mixer_simple_elem ( qsme );
				mspg->set_snd_dir ( snd_dir );
				mspg->set_group_name ( qsme->display_name() );

				if ( should_be_separated ( mspg ) ) {
					setup_proxies_group_separate ( mspg );
				} else {
					setup_proxies_group_joined ( mspg );
				}

				if ( mspg->proxies_sets().size() > 0 ) {
					_proxies_groups.append ( mspg );
				} else {
					delete mspg;
				}
			}
		}
	}

}


void
Mixer_Sliders::setup_proxies_group_joined (
	Mixer_Sliders_Proxies_Group * mspg_n )
{
	mspg_n->clear_proxies_sets();
	mspg_n->set_is_joined ( true );

	QString ttip;
	ttip = mspg_n->group_name();
	mspg_n->set_tool_tip ( ttip );

	create_proxies_set ( mspg_n, 0 );
	mspg_n->update_mixer_values();
}


void
Mixer_Sliders::setup_proxies_group_separate (
	Mixer_Sliders_Proxies_Group * mspg_n )
{

	const unsigned int num_channels (
		mspg_n->mixer_simple_elem()->num_channels ( mspg_n->snd_dir() ) );

	mspg_n->clear_proxies_sets();
	mspg_n->set_is_joined ( false );

	QString ttip;
	ttip = mspg_n->group_name();
	mspg_n->set_tool_tip ( ttip );

	for ( unsigned int ii=0; ii < num_channels; ++ii ) {
		create_proxies_set ( mspg_n, ii );
	}
	mspg_n->update_mixer_values();
}


bool
Mixer_Sliders::create_proxies_set (
	Mixer_Sliders_Proxies_Group * mspg_n,
	unsigned int channel_idx_n )
{
	bool res ( false );

	Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );
	const unsigned int snd_dir ( mspg_n->snd_dir() );

	Mixer_Sliders_Proxies_Set * pset (
		new Mixer_Sliders_Proxies_Set ( mspg_n ) );

	// Item name
	QString iname;
	if ( mspg_n->is_joined() ) {
		iname = mspg_n->group_name();
	} else {
		iname = tr ( "%1 (%2)" );
		iname = iname.arg ( QCoreApplication::translate (
			"ALSA::Channel_Name",
			qsme->channel_name ( snd_dir, channel_idx_n ) ) );
		iname = iname.arg ( qsme->channel_buffer ( snd_dir )[channel_idx_n] );
	}

	// Tool tip
	QString ttip_name;
	QString ttip_channel;
	ttip_name += "<div><b>";
	ttip_name += mspg_n->group_name();
	ttip_name += "</b></div>\n";

	if ( !mspg_n->is_joined() ) {
		ttip_channel += "<div>";
		ttip_channel += iname;
		ttip_channel += "</div>\n";
	}

	// Create volume proxy
	if ( qsme->has_volume ( snd_dir ) &&
		( channel_idx_n < qsme->volume_buffer ( snd_dir ).size() ) )
	{
		Mixer_Sliders_Proxy_Slider * msps (
			new Mixer_Sliders_Proxy_Slider ( mspg_n ) );

		msps->set_mixer_simple_elem ( qsme );
		msps->set_snd_dir ( snd_dir );
		msps->set_is_joined ( mspg_n->is_joined() );
		msps->set_channel_idx ( channel_idx_n );

		msps->set_item_name ( iname );
		msps->set_group_name ( mspg_n->group_name() );

		{
			QString ttip_type;
			ttip_type += "<div>";
			if ( snd_dir == 0 ) {
				ttip_type += tr ( "Playback slider" );
			} else {
				ttip_type += tr ( "Capture slider" );
			}
			ttip_type += "</div>";

			QString ttip ( ttip_name );
			ttip += ttip_type;
			ttip += ttip_channel;
			msps->set_tool_tip ( ttip );
		}


		msps->set_slider_range (
			qsme->volume_min ( snd_dir ),
			qsme->volume_max ( snd_dir ) );

		// Style info
		Wdg::Sliders_Pad_Proxy_Style_Info * sinfo (
			new Wdg::Sliders_Pad_Proxy_Style_Info ( snd_dir ) );

		if ( msps->has_dB() ) {
			if ( ( msps->dB_max() > 0 ) && ( msps->dB_min() < 0 ) ) {
				sinfo->slider_has_minimum = true;
				sinfo->slider_minimum_idx = msps->ask_dB_vol_nearest ( 0 );
			} else {
				if ( msps->dB_max() <= 0 ) {
					sinfo->slider_has_minimum = true;
					sinfo->slider_minimum_idx = msps->slider_value_max_idx();
				} else if ( msps->dB_min() >= 0 ) {
					sinfo->slider_has_minimum = true;
					sinfo->slider_minimum_idx = 0;
				}
			}
		}

		msps->set_style_info ( sinfo );

		pset->set_slider_proxy ( msps );
	}


	// Create switch proxy
	if ( qsme->has_switch ( snd_dir ) &&
		( channel_idx_n < qsme->switch_buffer ( snd_dir ).size() ) )
	{
		Mixer_Sliders_Proxy_Switch * msps (
			new Mixer_Sliders_Proxy_Switch ( mspg_n ) );

		msps->set_mixer_simple_elem ( qsme );
		msps->set_snd_dir ( snd_dir );
		msps->set_style_info (
			new Wdg::Sliders_Pad_Proxy_Style_Info ( snd_dir ) );
		msps->set_is_joined ( mspg_n->is_joined() );
		msps->set_channel_idx ( channel_idx_n );

		msps->set_item_name ( iname );

		{
			QString ttip_type;
			ttip_type += "<div>";
			if ( snd_dir == 0 ) {
				ttip_type += tr ( "Playback switch" );
			} else {
				ttip_type += tr ( "Capture switch" );
			}
			ttip_type += "</div>";

			QString ttip ( ttip_name );
			ttip += ttip_type;
			ttip += ttip_channel;
			msps->set_tool_tip ( ttip );
		}
		pset->set_switch_proxy ( msps );
	}

	if ( pset->has_slider() || pset->has_switch() ) {
		mspg_n->append_proxies_set ( pset );
	} else {
		delete pset;
		res = true;
	}

	return res;
}


void
Mixer_Sliders::update_visible_proxies_groups ( )
{
	_proxies_groups_visible.clear();

	for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
		Mixer_Sliders_Proxies_Group * mspg ( _proxies_groups[ii] );
		mspg->set_is_visible ( should_be_visible ( mspg ) );
		if ( mspg->is_visible() ) {
			_proxies_groups_visible.append ( mspg );
		}
	}
}


void
Mixer_Sliders::update_layout ( )
{
	//std::cout << "Mixer_Sliders::update_layout" << "\n";

	sliders_pad()->set_proxies_groups ( 0 );

	update_visible_proxies_groups();
	update_visible_mixer_values();

	sliders_pad()->set_proxies_groups ( &_proxies_groups_visible );
}


void
Mixer_Sliders::action_level_volumes ( )
{
	//std::cout << "Mixer_Sliders::action_level_volumes" << "\n";

	Mixer_Sliders_Proxies_Group * mspg ( 0 );
	Mixer_Sliders_Proxies_Set * msps ( 0 );

	context_menu_proxies_group ( &mspg, &msps );
	_context_menu_focus.clear();

	if ( mspg != 0 ) {
		int wdgs_idx ( 0 );
		{
			const Wdg::Pad_Focus_Info & fi ( sliders_pad()->focus_info() );
			if ( fi.has_focus ) {
				if ( fi.widgets_idx < mspg->num_sliders() ) {
					wdgs_idx =  fi.widgets_idx;
				}
			}
		}

		Wdg::Sliders_Pad_Proxy_Slider * psl ( 0 );
		if ( msps != 0 ) {
			psl = msps->slider_proxy();
		}

		if ( psl != 0 ) {
			mspg->mixer_simple_elem()->set_volume_all (
				mspg->snd_dir(), psl->slider_value() );
		} else {
			mspg->mixer_simple_elem()->level_volumes ( mspg->snd_dir() );
		}
	}
}


void
Mixer_Sliders::action_toggle_joined_separated ( )
{
	//std::cout << "Mixer_Sliders::action_toggle_joined_separated" << "\n";

	Mixer_Sliders_Proxies_Group * mspg ( 0 );
	context_menu_proxies_group ( &mspg );

	if ( mspg != 0 ) {
		_context_menu_focus.clear();
		toggle_joined_separated ( mspg );
	}
}


void
Mixer_Sliders::action_mute_all ( )
{
	//std::cout << "Mixer_Sliders::action_mute_all" << "\n";

	Mixer_Sliders_Proxies_Group * mspg ( 0 );
	context_menu_proxies_group ( &mspg );
	if ( mspg != 0 ) {
		_context_menu_focus.clear();
		mspg->mixer_simple_elem()->set_switch_all ( mspg->snd_dir(), false );
	}
}


void
Mixer_Sliders::action_unmute_all ( )
{
	//std::cout << "Mixer_Sliders::action_unmute_all" << "\n";

	Mixer_Sliders_Proxies_Group * mspg ( 0 );
	context_menu_proxies_group ( &mspg );
	if ( mspg != 0 ) {
		_context_menu_focus.clear();
		mspg->mixer_simple_elem()->set_switch_all ( mspg->snd_dir(), true );
	}
}



void
Mixer_Sliders::acquire_focus_proxy (
	Mixer_Proxy_Selection & sel_n )
{
	const Wdg::Pad_Focus_Info & fi ( sliders_pad()->focus_info() );
	if ( fi.has_focus ) {
		Mixer_Sliders_Proxies_Group * mspg (
			proxies_group ( _proxies_groups_visible[fi.group_idx] ) );
		sel_n.set_group_name ( mspg->group_name() );
		sel_n.set_snd_dir ( mspg->snd_dir() );
		sel_n.set_proxy_idx ( fi.widgets_idx );
	}
}


void
Mixer_Sliders::set_focus_proxy (
	const Mixer_Proxy_Selection & sel_n )
{
	if ( sel_n.is_clear() ) {
		return;
	}

	for ( int ii=0; ii < _proxies_groups_visible.size(); ++ii ) {
		const Mixer_Sliders_Proxies_Group * mspg2 (
			proxies_group ( _proxies_groups_visible[ii] ) );
		if ( ( mspg2->group_name() == sel_n.group_name() ) &&
			( mspg2->snd_dir() == sel_n.snd_dir() ) )
		{
			sliders_pad()->set_focus_proxy ( ii, sel_n.proxy_idx() );
			break;
		}
	}
}


void
Mixer_Sliders::toggle_joined_separated (
	Mixer_Sliders_Proxies_Group * mspg_n )
{
	if ( mspg_n != 0 ) {

		if ( !mspg_n->can_be_separate() ) {
			return;
		}

		Mixer_Proxy_Selection focus_sel;
		acquire_focus_proxy ( focus_sel );
		sliders_pad()->set_proxies_groups ( 0 );

		if ( mspg_n->is_joined() ) {
			proxies_group_separate ( mspg_n );
		} else {
			proxies_group_join ( mspg_n );
		}

		sliders_pad()->set_proxies_groups ( &_proxies_groups_visible );
		set_focus_proxy ( focus_sel );
	}
}


void
Mixer_Sliders::proxies_group_join (
	Wdg::Sliders_Pad_Proxies_Group * sppg_n )
{
	Mixer_Sliders_Proxies_Group * mspg ( proxies_group ( sppg_n ) );

	if ( !mspg->is_joined() ) {
		Snd_Mixer_Simple_Elem * qsme ( mspg->mixer_simple_elem() );
		qsme->level_volumes ( mspg->snd_dir() );
		qsme->level_switches ( mspg->snd_dir() );

		setup_proxies_group_joined ( mspg );
		if ( _context_menu.isVisible() ) {
			context_menu_update();
		}
	}
}


void
Mixer_Sliders::proxies_group_separate (
	Wdg::Sliders_Pad_Proxies_Group * sppg_n )
{
	Mixer_Sliders_Proxies_Group * mspg ( proxies_group ( sppg_n ) );

	if ( mspg->can_be_separate() ) {
		setup_proxies_group_separate ( mspg );
		if ( _context_menu.isVisible() ) {
			context_menu_update();
		}
	}
}


void
Mixer_Sliders::update_visible_mixer_values ( )
{
	for ( int ii=0; ii < _proxies_groups_visible.size(); ++ii ) {
		proxies_group ( _proxies_groups_visible[ii] )->update_mixer_values();
	}
}


void
Mixer_Sliders::mixer_values_changed ( )
{
	if ( _mixer_simple != 0 ) {
		evaluate_mixer_values();
		update_visible_mixer_values();
		if ( _context_menu.isVisible() ) {
			context_menu_update();
		}
	}
}


void
Mixer_Sliders::evaluate_mixer_values ( )
{
	bool vis_change ( false );
	bool sep_needed ( false );
	QTime time_now ( QTime::currentTime() );

	//std::cout << "Mixer_Sliders::evaluate_mixer_values" << "\n";

	// Not matching visibility check (is_active flag)
	for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
		Mixer_Sliders_Proxies_Group * mspg ( _proxies_groups[ii] );
		if ( should_be_visible ( mspg ) != mspg->is_visible() ) {
			vis_change = true;
			break;
		}
	}

	// Check for not equal channel values
	for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
		Mixer_Sliders_Proxies_Group * mspg ( _proxies_groups[ii] );
		if ( needs_separation ( mspg ) ) {
			Mixer_Separation_Info & sep_info ( mspg->separation_info() );
			if ( !sep_info.requested() ) {
				sep_info.set_requested ( true );
				sep_info.set_time ( time_now );
				sep_needed = true;
			}
		}
	}

	if ( vis_change ) {
		Mixer_Proxy_Selection focus_sel;
		acquire_focus_proxy ( focus_sel );
		sliders_pad()->set_proxies_groups ( 0 );

		update_visible_proxies_groups();

		sliders_pad()->set_proxies_groups ( &_proxies_groups_visible );
		set_focus_proxy ( focus_sel );
	}

	if ( sep_needed ) {
		if ( !_timer_separation.isActive() ) {
			_timer_separation.start();
		}
	}
}



void
Mixer_Sliders::timed_separation_check ( )
{
	const int sep_msecs ( 550 );
	QTime time_now ( QTime::currentTime() );

	bool keep_timer_running ( false );
	bool vis_sep_needed ( false );

	// Separate offscreen proxy groups and check for visible separation needs
	for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
		Mixer_Sliders_Proxies_Group * mspg ( _proxies_groups[ii] );
		Mixer_Separation_Info & sep_info ( mspg->separation_info() );
		if ( sep_info.requested() ) {
			if ( needs_separation ( mspg ) ) {
				if ( sep_info.time().msecsTo ( time_now ) < sep_msecs ) {
					// Delay separation further
					keep_timer_running = true;
				} else {
					// Separate NOW
					if ( mspg->is_visible() ) {
						sep_info.set_do_it ( true );
						vis_sep_needed = true;
					} else {
						proxies_group_separate ( mspg );
					}
					sep_info.set_requested ( false );
				}
			} else {
				sep_info.set_requested ( false );
			}
		}
	}

	if ( vis_sep_needed ) {
		Mixer_Proxy_Selection focus_sel;
		acquire_focus_proxy ( focus_sel );
		sliders_pad()->set_proxies_groups ( 0 );

		for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
			Mixer_Sliders_Proxies_Group * mspg ( _proxies_groups[ii] );
			Mixer_Separation_Info & sep_info ( mspg->separation_info() );
			if ( sep_info.do_it() ) {
				sep_info.set_do_it ( false );
				proxies_group_separate ( mspg );
			}
		}

		sliders_pad()->set_proxies_groups ( &_proxies_groups_visible );
		set_focus_proxy ( focus_sel );
	}

	if ( !keep_timer_running ) {
		_timer_separation.stop();
	}
}


void
Mixer_Sliders::context_menu_start (
	const QPoint & pos_n )
{
	if ( _context_menu.isVisible() ) {
		return;
	}

	_context_menu_focus.clear();
	_actions_visible = 0;

	if ( !sliders_pad()->focus_info().has_focus ) {
		return;
	}

	const unsigned int & idx ( sliders_pad()->focus_info().group_idx );
	Mixer_Sliders_Proxies_Group * mspg (
		proxies_group ( _proxies_groups_visible[idx] ) );

	_context_menu_focus.set_group_name ( mspg->group_name() );
	_context_menu_focus.set_snd_dir ( mspg->snd_dir() );
	_context_menu.setTitle ( _context_menu_focus.group_name() );

	context_menu_update();

	if ( _actions_visible > 0 ) {
		_context_menu.popup ( pos_n );
	}
}


void
Mixer_Sliders::context_menu_proxies_group (
	Mixer_Sliders_Proxies_Group * * mspg_n,
	Mixer_Sliders_Proxies_Set * * msps_n )
{
	Mixer_Sliders_Proxies_Group * mspg ( 0 );

	int prox_idx ( 0 );

	// Take the current focus group first
	if ( sliders_pad()->focus_info().has_focus ) {
		const int & grp_idx ( sliders_pad()->focus_info().group_idx );
		if ( grp_idx < _proxies_groups_visible.size() ) {
			mspg = proxies_group ( _proxies_groups_visible[grp_idx] );
			prox_idx = sliders_pad()->focus_info().widgets_idx;
		}
	}

	// Take the previously selected group on demand
	if ( ( mspg == 0 ) && ( !_context_menu_focus.is_clear() ) ) {
		prox_idx = _context_menu_focus.proxy_idx();

		// Find visible group by name
		for ( int ii=0; ii < _proxies_groups_visible.size(); ++ii ) {
			mspg = proxies_group ( _proxies_groups_visible[ii] );
			if ( ( mspg->group_name() == _context_menu_focus.group_name() ) &&
				( mspg->snd_dir() == _context_menu_focus.snd_dir() ) )
			{
				break;
			}
			mspg = 0;
		}
	}

	if ( ( mspg != 0 ) && ( msps_n != 0 ) ) {
		if ( prox_idx >= mspg->proxies_sets().size() ) {
			prox_idx = 0;
		}
		if ( prox_idx < mspg->proxies_sets().size() ) {
			*msps_n = dynamic_cast < Mixer_Sliders_Proxies_Set * > (
				mspg->proxies_set ( prox_idx ) );
		} else {
			*msps_n = 0;
		}
	}

	*mspg_n = mspg;
}


void
Mixer_Sliders::context_menu_update ( )
{
	_actions_visible = 0;

	Mixer_Sliders_Proxies_Group * mspg ( 0 );
	context_menu_proxies_group ( &mspg );

	if ( mspg == 0 ) {
		_context_menu.close();
		return;
	}

	const Snd_Mixer_Simple_Elem * qsme ( mspg->mixer_simple_elem() );

	// Split/Join and level channels
	{
		const bool vis_separate ( mspg->can_be_separate() );
		const bool vis_level ( mspg->num_sliders() > 1 );

		_act_separate_join.setVisible ( vis_separate );
		_act_level_volumes.setVisible ( vis_level );
		_act_separator_channels.setVisible ( vis_separate || vis_level );
		if ( vis_separate ) {
			++_actions_visible;
			if ( mspg->is_joined() ) {
				_act_separate_join.setText ( _act_str_toggle_separate[0] );
			} else {
				_act_separate_join.setText ( _act_str_toggle_separate[1] );
			}
		}
		if ( vis_level ) {
			++_actions_visible;
			bool enabled ( !qsme->volumes_equal ( mspg->snd_dir() ) );
			_act_level_volumes.setEnabled ( enabled );
		}
	}


	// Mute / unmute
	{
		bool vis_mute ( mspg->num_switches() > 0 );
		bool vis_unmute ( vis_mute );
		bool enable_mute ( true );
		bool enable_unmute ( true );

		if ( vis_mute ) {
			++_actions_visible;
			if ( mspg->proxies_sets().size() == 1 ) {
				_act_mute_all.setText ( _act_str_mute[0] );
				_act_unmute_all.setText ( _act_str_unmute[0] );
				if ( qsme->switch_buffer ( mspg->snd_dir() )[0] ) {
					vis_unmute = false;
				} else {
					vis_mute = false;
				}
			} else {
				++_actions_visible;
				_act_mute_all.setText ( _act_str_mute[1] );
				_act_unmute_all.setText ( _act_str_unmute[1] );
				if ( qsme->switches_equal ( mspg->snd_dir() ) ) {
					enable_mute = qsme->switch_buffer ( mspg->snd_dir() )[0];
					enable_unmute = !enable_mute;
				}
			}
		}

		_act_mute_all.setVisible ( vis_mute );
		_act_mute_all.setEnabled ( enable_mute );
		_act_unmute_all.setVisible ( vis_unmute );
		_act_unmute_all.setEnabled ( enable_unmute );
	}

	if ( _actions_visible == 0 ) {
		_context_menu.close();
	}
}


bool
Mixer_Sliders::eventFilter (
	QObject * watched,
	QEvent * event )
{
	bool res ( false );
	Wdg::Sliders_Pad * sp ( dynamic_cast < Wdg::Sliders_Pad * > ( watched ) );
	if ( sp != 0 ) {
		if ( event->type() == QEvent::KeyPress ) {
			QKeyEvent * kev ( dynamic_cast < QKeyEvent * > ( event ) );
			if ( kev != 0 ) {
				res = true;
				const QKeySequence key_seq ( kev->key() );
				if ( _act_separate_join.shortcut() == key_seq ) {
					_act_separate_join.trigger();
				} else if ( _act_level_volumes.shortcut() == key_seq ) {
					_act_level_volumes.trigger();
				} else if ( _act_mute_all.shortcut() == key_seq ) {
					_act_mute_all.trigger();
				} else if ( _act_unmute_all.shortcut() == key_seq ) {
					_act_unmute_all.trigger();
				} else {
					res = false;
				}
			}
		} else if ( event->type() == QEvent::ContextMenu ) {
			QContextMenuEvent * cmev ( dynamic_cast < QContextMenuEvent * > ( event ) );
			if ( cmev != 0 ) {
				context_menu_start ( cmev->globalPos() );
				res = true;
			}
		}
	}

	return res;
}


} // End of namespace

