//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "mixer_switches.hpp"

#include <QHBoxLayout>
#include <QVBoxLayout>

#include <QCoreApplication>
#include <QEvent>
#include <QKeyEvent>
#include <QContextMenuEvent>

#include <iostream>


namespace QSnd
{


//
// Mixer_Switches
//
Mixer_Switches::Mixer_Switches (
	QWidget * parent_n ) :
QWidget ( parent_n ),
_pixmaps_hub ( 0 ),
_act_separate_join ( this ),
_mixer ( 0 )
{
	_sw_block[0].set_title ( tr ( "Playback switches" ) );
	_sw_block[0].set_tool_tip ( tr ( "Playback switches" ) );

	_sw_block[1].set_title ( tr ( "Capture switches" ) );
	_sw_block[1].set_tool_tip ( tr ( "Capture switches" ) );

	for ( int ii=0; ii < 2; ++ii ) {
		_sw_block[ii].switches_pad().installEventFilter ( this );
	}


	// Actions
	_act_separate_join.setShortcut ( Qt::Key_C );

	_act_str_separate_join[0] = tr ( "Split channels" );
	_act_str_separate_join[1] = tr ( "Join channels" );

	connect ( &_act_separate_join, SIGNAL ( triggered ( bool ) ),
		this, SLOT ( action_toggle_joined_separated() ) );

	// Context menu
	_context_menu.addAction ( &_act_separate_join );


	// Separation timer
	_timer_separation.setSingleShot ( false );
	_timer_separation.setInterval ( 200 );
	connect ( &_timer_separation, SIGNAL ( timeout() ),
		this, SLOT ( timed_separation_check() ) );


	// Center widget layout (inside the scroll area)
	QHBoxLayout * lay_hbox = new QHBoxLayout();
	lay_hbox->setContentsMargins ( 0, 0, 0, 0 );

	// Playback switches
	const Qt::Alignment align_tl ( Qt::AlignLeft | Qt::AlignTop );
	lay_hbox->setSpacing ( 11 );
	lay_hbox->addWidget ( &_sw_block[0], 0, align_tl );
	lay_hbox->addWidget ( &_sw_block[1], 0, align_tl );
	lay_hbox->addStretch ( 1 );

	{
		QSizePolicy policy ( sizePolicy() );
		policy.setHorizontalPolicy ( QSizePolicy::Fixed );
		policy.setVerticalPolicy ( QSizePolicy::Fixed );
		setSizePolicy ( policy );
	}

	setLayout ( lay_hbox );

	update_layout();
}


Mixer_Switches::~Mixer_Switches ( )
{
	set_mixer_simple ( 0 );
}


bool
Mixer_Switches::has_visible_blocks ( ) const
{
	for ( int ii=0; ii < 2; ++ii ) {
		if ( has_switches ( ii ) && _config.show_dir[ii] ) {
			return true;
		}
	}
	return false;
}


void
Mixer_Switches::set_mixer_simple (
	Snd_Mixer_Simple * mixer_n  )
{
	if ( mixer_n == _mixer ) {
		return;
	}

	emit sig_layout_about_to_change();

	if ( _mixer != 0 ) {
		disconnect ( _mixer, 0, this, 0 );
	}

	for ( int ii=0; ii < 2; ++ii ) {
		_sw_block[ii].switches_pad().set_proxies_groups ( 0 );
	}
	clear_proxies_groups();

	_mixer = mixer_n;

	if ( _mixer != 0 ) {

		connect ( _mixer, SIGNAL ( sig_mixer_values_changed() ),
			this, SLOT ( mixer_values_changed() ) );

		create_proxies_groups();
		update_visible_proxies_groups();
		update_visible_mixer_values();

		for ( int ii=0; ii < 2; ++ii ) {
			_sw_block[ii].switches_pad().set_proxies_groups (
				&_proxies_groups_visible[ii] );
		}
	}

	update_layout();

	emit sig_layout_changed();
}


void
Mixer_Switches::set_mixer_config (
	const Mixer_Config & cfg_n )
{
	if ( !_config.show_dirs_equal ( cfg_n ) ) {
		emit sig_layout_about_to_change();

		_config = cfg_n;
		update_layout();

		emit sig_layout_changed();
	}
}


void
Mixer_Switches::clear_proxies_groups ( )
{
	for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
		delete _proxies_groups[ii];
	}
	_proxies_groups.clear();

	for ( int ii=0; ii < 2; ++ii ) {
		_proxies_groups_dir[ii].clear();
	}
	for ( int ii=0; ii < 2; ++ii ) {
		_proxies_groups_visible[ii].clear();
	}
}


void
Mixer_Switches::create_proxies_groups ( )
{
	if ( _mixer == 0 ) {
		return;
	}

	//std::cout << "Mixer_Switches::init_widgets\n";

	QString ttip;

	for ( int ii=0; ii < _mixer->elems().size(); ++ii ) {
		Snd_Mixer_Simple_Elem * qsme ( _mixer->elems()[ii] );

		// Types means either enum or switch
		for ( unsigned int itype=0; itype < 2; ++itype ) {
			for ( unsigned int snd_dir=0; snd_dir < 2; ++snd_dir ) {
				if ( qsme->has_volume( snd_dir ) ) {
					continue;
				}

				bool cr_enum   ( itype == 0 );
				bool cr_switch ( itype == 1 );

				if ( ( cr_enum && qsme->has_enum ( snd_dir ) ) ||
					( cr_switch && qsme->has_switch ( snd_dir ) ) )
				{
					Mixer_Switches_Proxies_Group * mspg (
						new Mixer_Switches_Proxies_Group ( this ) );
					mspg->set_mixer_simple_elem ( qsme );
					mspg->set_snd_dir ( snd_dir );
					mspg->set_group_name ( qsme->display_name() );
					mspg->set_is_enum ( cr_enum );
					mspg->set_is_switch ( cr_switch );

					{
						ttip = "<div><b>";
						ttip += mspg->group_name();
						ttip += "</b></div>\n";
						mspg->set_tool_tip ( ttip );
					}

					if ( should_be_separated ( mspg ) ) {
						setup_proxies_group_separate ( mspg );
					} else {
						setup_proxies_group_joined ( mspg );
					}

					if ( mspg->proxies().size() > 0 ) {
						_proxies_groups.append ( mspg );
						_proxies_groups_dir[snd_dir].append ( mspg );
					} else {
						delete mspg;
					}
				}
			}
		}

	}

}


void
Mixer_Switches::setup_proxies_group_joined (
	Mixer_Switches_Proxies_Group * mspg_n )
{
	const Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );
	const unsigned int snd_dir ( mspg_n->snd_dir() );

	mspg_n->clear_proxies();
	mspg_n->set_is_joined ( true );

	unsigned int num_channels ( 0 );
	if ( mspg_n->is_switch() ) {
		num_channels = qsme->switch_buffer ( snd_dir ).size();
	} else if ( mspg_n->is_enum() ) {
		num_channels = qsme->enum_buffer ( snd_dir ).size();
	}
	if ( num_channels > 0 ) {
		Mixer_Switches_Proxy * msp (
			create_proxy ( mspg_n, 0 ) );

		if ( msp != 0 ) {
			mspg_n->append_proxy ( msp );
		}
	}

	mspg_n->update_mixer_values();
}


void
Mixer_Switches::setup_proxies_group_separate (
	Mixer_Switches_Proxies_Group * mspg_n )
{
	const Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );
	const unsigned int snd_dir ( mspg_n->snd_dir() );

	mspg_n->clear_proxies();
	mspg_n->set_is_joined ( false );

	unsigned int num_channels ( 0 );
	if ( mspg_n->is_switch() ) {
		num_channels = qsme->switch_buffer ( snd_dir ).size();
	} else if ( mspg_n->is_enum() ) {
		num_channels = qsme->enum_buffer ( snd_dir ).size();
	}

	for ( unsigned int ii=0; ii < num_channels; ++ii ) {
		Mixer_Switches_Proxy * msp ( create_proxy ( mspg_n, ii ) );
		if ( msp != 0 ) {
			mspg_n->append_proxy ( msp );
		}
	}

	mspg_n->update_mixer_values();
}


Mixer_Switches_Proxy *
Mixer_Switches::create_proxy (
	Mixer_Switches_Proxies_Group * mspg_n,
	int channel_idx_n )
{
	Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );
	const unsigned int snd_dir ( mspg_n->snd_dir() );

	QString iname;
	if ( mspg_n->is_joined() ) {
		iname = mspg_n->group_name();
	} else {
		iname = tr ( "%1 (%2)" );
		iname = iname.arg ( QCoreApplication::translate (
			"ALSA::Channel_Name",
			qsme->channel_name ( snd_dir, channel_idx_n ) ) );
		iname = iname.arg ( qsme->channel_buffer ( snd_dir )[channel_idx_n] );
	}

	QString ttip;
	ttip += "<div><b>";
	ttip += mspg_n->group_name();
	ttip += "</b></div>\n";

	ttip += "<div>";
	if ( snd_dir == 0 ) {
		if ( mspg_n->is_enum() ) {
			ttip += tr ( "Playback selection" );
		} else {
			ttip += tr ( "Playback switch" );
		}
	} else {
		if ( mspg_n->is_enum() ) {
			ttip += tr ( "Capture selection" );
		} else {
			ttip += tr ( "Capture switch" );
		}
	}
	ttip += "</div>\n";

	if ( !mspg_n->is_joined() ) {
		ttip += "<div>";
		ttip += iname;
		ttip += "</div>";
	}

	Mixer_Switches_Proxy * msp (
		new Mixer_Switches_Proxy ( mspg_n ) );
	msp->set_mixer_simple_elem ( qsme );
	msp->set_snd_dir ( snd_dir );
	msp->set_is_joined ( mspg_n->is_joined() );
	msp->set_is_enum ( mspg_n->is_enum() );
	msp->set_is_switch ( mspg_n->is_switch() );
	msp->set_channel_idx ( channel_idx_n );

	msp->set_item_name ( iname );
	msp->set_tool_tip ( ttip );

	if ( msp->is_enum() ) {
		msp->set_enum_num_items ( qsme->enum_item_names().size() );
	}

	return msp;
}


void
Mixer_Switches::update_visible_proxies_group (
	int snd_dir_n )
{
	_proxies_groups_visible[snd_dir_n].clear();

	Mixer_Switches_Proxy_Group_List & groups (
		_proxies_groups_dir[snd_dir_n] );

	Wdg::Switches_Pad_Proxy_Group_List & pg_vis (
		_proxies_groups_visible[snd_dir_n] );

	for ( int ii=0; ii < groups.size(); ++ii ) {
		Mixer_Switches_Proxies_Group * mspg ( groups[ii] );
		mspg->set_is_visible ( should_be_visible ( mspg ) );
		if ( mspg->is_visible() ) {
			pg_vis.append ( mspg );
		}
	}
}


void
Mixer_Switches::update_visible_proxies_groups ( )
{
	for ( int snd_dir=0; snd_dir < 2; ++snd_dir ) {
		update_visible_proxies_group ( snd_dir );
	}
}


void
Mixer_Switches::update_layout ( )
{
	//std::cout << "Mixer_Switches::update_layout\n";

	for ( int snd_dir=0; snd_dir < 2; ++snd_dir ) {
		const bool show_block (
			_config.show_dir[snd_dir] && has_switches ( snd_dir ) );
		_sw_block[snd_dir].setVisible ( show_block );
	}

	updateGeometry();
	adjustSize();
}


void
Mixer_Switches::update_visible_mixer_values ( )
{
	for ( int ii=0; ii < 2; ++ii ) {
		if ( _config.show_dir[ii] ) {
			const Wdg::Switches_Pad_Proxy_Group_List & pg (
				_proxies_groups_visible[ii] );
			for ( int jj=0; jj < pg.size(); ++jj ) {
				proxies_group ( pg[jj] )->update_mixer_values();
			}
		}
	}
}


Wdg::Switches_Pad *
Mixer_Switches::acquire_focus_pad ( )
{
	Wdg::Switches_Pad * sw_pad ( 0 );
	for ( int ii=0; ii < 2; ++ii ) {
		if ( _sw_block[ii].switches_pad().focus_info().has_focus ) {
			sw_pad = &_sw_block[ii].switches_pad();
			break;
		}
	}
	return sw_pad;
}


void
Mixer_Switches::acquire_focus_proxy (
	Wdg::Switches_Pad * sw_pad_n,
	Mixer_Proxy_Selection & sel_n )
{
	if ( sw_pad_n == 0 ) {
		return;
	}

	Wdg::Switches_Pad_Proxy_Group_List & pg ( *sw_pad_n->proxies_groups() );
	const Wdg::Pad_Focus_Info & fi ( sw_pad_n->focus_info() );
	if ( fi.has_focus ) {
		const Mixer_Switches_Proxies_Group * mspg (
			proxies_group ( pg[fi.group_idx] ) );
		sel_n.set_group_name ( mspg->group_name() );
		sel_n.set_snd_dir ( mspg->snd_dir() );
		sel_n.set_proxy_idx ( fi.widgets_idx );
	}
}


void
Mixer_Switches::set_focus_proxy (
	Wdg::Switches_Pad * sw_pad_n,
	const Mixer_Proxy_Selection & sel_n )
{
	if ( ( sw_pad_n == 0 ) || sel_n.is_clear() ) {
		return;
	}

	Wdg::Switches_Pad_Proxy_Group_List & groups (
		*sw_pad_n->proxies_groups() );

	for ( int ii=0; ii < groups.size(); ++ii ) {
		const Mixer_Switches_Proxies_Group * mspg (
			proxies_group ( groups[ii] ) );
		if ( ( mspg->group_name() == sel_n.group_name() ) &&
		     ( mspg->snd_dir()    == sel_n.snd_dir() ) )
		{
			sw_pad_n->set_focus_proxy ( ii, sel_n.proxy_idx() );
			break;
		}
	}
}


void
Mixer_Switches::action_toggle_joined_separated ( )
{
	//std::cout << "Mixer_Switches::action_toggle_joined_separated" << "\n";

	Wdg::Switches_Pad * focus_pad ( acquire_focus_pad() );
	Mixer_Switches_Proxies_Group * mspg (
		context_menu_proxies_group ( focus_pad ) );

	if ( mspg != 0 ) {
		_context_menu_focus.clear();
		toggle_joined_separated ( focus_pad, mspg );
	}
}


void
Mixer_Switches::toggle_joined_separated (
	Wdg::Switches_Pad * pad_n,
	Mixer_Switches_Proxies_Group * mspg_n )
{
	if ( ( pad_n != 0 ) && ( mspg_n != 0 ) ) {

		if ( !mspg_n->can_be_separate() ) {
			return;
		}

		Mixer_Proxy_Selection focus_sel;
		acquire_focus_proxy ( pad_n, focus_sel );

		emit sig_layout_about_to_change();

		Wdg::Switches_Pad_Proxy_Group_List * pg (
			pad_n->proxies_groups() );

		pad_n->set_proxies_groups ( 0 );

		if ( mspg_n->is_joined() ) {
			proxies_group_separate ( mspg_n );
		} else {
			proxies_group_join ( mspg_n );
		}

		pad_n->set_proxies_groups ( pg );
		updateGeometry();

		emit sig_layout_changed();

		set_focus_proxy ( pad_n, focus_sel );
	}
}


void
Mixer_Switches::proxies_group_join (
	Mixer_Switches_Proxies_Group * mspg_n )
{
	if ( !mspg_n->is_joined() ) {
		Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );
		qsme->level_switches ( mspg_n->snd_dir() );

		setup_proxies_group_joined ( mspg_n );
		if ( _context_menu.isVisible() ) {
			context_menu_update();
		}
	}
}


void
Mixer_Switches::proxies_group_separate (
	Mixer_Switches_Proxies_Group * mspg_n )
{
	if ( mspg_n->is_joined() && mspg_n->can_be_separate() ) {
		setup_proxies_group_separate ( mspg_n );
		if ( _context_menu.isVisible() ) {
			context_menu_update();
		}
	}
}


void
Mixer_Switches::mixer_values_changed ( )
{
	if ( _mixer == 0 ) {
		return;
	}

	//std::cout << "Mixer_Switches::mixer_values_changed" << "\n";

	evaluate_mixer_values();
	update_visible_mixer_values();
	if ( _context_menu.isVisible() ) {
		context_menu_update();
	}
}


void
Mixer_Switches::evaluate_mixer_values ( )
{
	bool vis_change[2] = { false, false };
	bool sep_needed ( false );
	QTime time_now ( QTime::currentTime() );

	//std::cout << "Mixer_Switches::evaluate_mixer_values" << "\n";

	for ( int snd_dir=0; snd_dir < 2; ++ snd_dir ) {

		Mixer_Switches_Proxy_Group_List & groups (
			_proxies_groups_dir[snd_dir] );

		// Check for not matching visibility (is_active flag)
		for ( int ii=0; ii < groups.size(); ++ii ) {
			Mixer_Switches_Proxies_Group * mspg ( groups[ii] );
			if ( should_be_visible ( mspg ) != mspg->is_visible() ) {
				vis_change[snd_dir] = true;
				break;
			}
		}

		// Check for not equal channel values
		for ( int ii=0; ii < groups.size(); ++ii ) {
			Mixer_Switches_Proxies_Group * mspg ( groups[ii] );
			if ( needs_separation ( mspg ) ) {
				Mixer_Separation_Info & sep_info ( mspg->separation_info() );
				if ( !sep_info.requested() ) {
					sep_info.set_requested ( true );
					sep_info.set_time ( time_now );
					sep_needed = true;
				}
			}
		}
	}


	// Update groups visibility immediately
	if ( vis_change[0] || vis_change[1] ) {

		// Save focus
		Mixer_Proxy_Selection focus_sel;
		for ( int snd_dir=0; snd_dir < 2; ++ snd_dir ) {
			acquire_focus_proxy ( switches_pad ( snd_dir ), focus_sel );
		}

		emit sig_layout_about_to_change();

		// Unset sliders pad proxies groups
		// Update visibility
		// Restore sliders pad proxies groups
		for ( int snd_dir=0; snd_dir < 2; ++ snd_dir ) {
			if ( vis_change[snd_dir] ) {
				switches_pad ( snd_dir )->set_proxies_groups ( 0 );

				update_visible_proxies_group ( snd_dir );

				switches_pad ( snd_dir )->set_proxies_groups (
					&_proxies_groups_visible[snd_dir] );
			}
		}

		emit sig_layout_changed();

		// Restore focus
		for ( int snd_dir=0; snd_dir < 2; ++ snd_dir ) {
			set_focus_proxy ( switches_pad ( snd_dir ), focus_sel );
		}
	}

	if ( sep_needed ) {
		if ( !_timer_separation.isActive() ) {
			_timer_separation.start();
		}
	}
}


void
Mixer_Switches::timed_separation_check ( )
{
	const int sep_msecs ( 350 );
	bool keep_timer_running ( false );
	bool vis_sep_needed[2] = { false, false };
	QTime time_now ( QTime::currentTime() );

	for ( int snd_dir=0; snd_dir < 2; ++ snd_dir ) {

		Mixer_Switches_Proxy_Group_List & groups (
			_proxies_groups_dir[snd_dir] );

		// Check for channel separation (not equal channel values)
		// Separate offscreen proxies groups silently
		for ( int ii=0; ii < groups.size(); ++ii ) {
			Mixer_Switches_Proxies_Group * mspg ( groups[ii] );
			Mixer_Separation_Info & sep_info ( mspg->separation_info() );
			if ( sep_info.requested() ) {
				if ( needs_separation ( mspg ) ) {
					if ( sep_info.time().msecsTo ( time_now ) < sep_msecs ) {
						// Delay separation further
						keep_timer_running = true;
					} else {
						// Separate NOW
						if ( mspg->is_visible() ) {
							sep_info.set_do_it ( true );
							vis_sep_needed[snd_dir] = true;
						} else {
							proxies_group_separate ( mspg );
						}
						sep_info.set_requested ( false );
					}
				} else {
					sep_info.set_requested ( false );
				}
			}
		}
	}

	if ( vis_sep_needed[0] || vis_sep_needed[1] ) {

		// Save focus
		Mixer_Proxy_Selection focus_sel;
		for ( int snd_dir=0; snd_dir < 2; ++snd_dir ) {
			acquire_focus_proxy ( switches_pad ( snd_dir ), focus_sel );
		}

		emit sig_layout_about_to_change();

		// Unset sliders pad proxies groups
		// separate on demand
		// Restore sliders pad proxies groups
		for ( int snd_dir=0; snd_dir < 2; ++snd_dir ) {
			if ( vis_sep_needed[snd_dir] ) {
				switches_pad ( snd_dir )->set_proxies_groups ( 0 );

				Mixer_Switches_Proxy_Group_List & groups (
					_proxies_groups_dir[snd_dir] );

				for ( int ii=0; ii < groups.size(); ++ii ) {
					Mixer_Switches_Proxies_Group * mspg ( groups[ii] );
					Mixer_Separation_Info & sep_info ( mspg->separation_info() );
					if ( sep_info.do_it() ) {
						sep_info.set_do_it ( false );
						proxies_group_separate ( mspg );
					}
				}

				switches_pad ( snd_dir )->set_proxies_groups (
					&_proxies_groups_visible[snd_dir] );
			}
		}

		emit sig_layout_changed();

		// Restore focus
		for ( int snd_dir=0; snd_dir < 2; ++ snd_dir ) {
			set_focus_proxy ( switches_pad ( snd_dir ), focus_sel );
		}
	}

	if ( !keep_timer_running ) {
		_timer_separation.stop();
	}
}


void
Mixer_Switches::context_menu_start (
	Wdg::Switches_Pad * sw_pad_n,
	const QPoint & pos_n )
{
	if ( _context_menu.isVisible() ) {
		//std::cout << "Mixer_Switches::context_menu_start: Menu is visible\n";
		return;
	}

	_context_menu_focus.clear();
	_actions_visible = 0;

	if ( !sw_pad_n->focus_info().has_focus ) {
		return;
	}

	// Find focus proxies_group
	Mixer_Switches_Proxies_Group * mspg ( 0 );
	{
		const int & idx ( sw_pad_n->focus_info().group_idx );
		Wdg::Switches_Pad_Proxy_Group_List * lst ( 0 );
		for ( int ii=0; ii < 2; ++ii ) {
			if ( sw_pad_n == &_sw_block[ii].switches_pad() ) {
				lst = &_proxies_groups_visible[ii];
				break;
			}
		}
		if ( lst != 0 ) {
			if ( idx < lst->size() ) {
				mspg = proxies_group ( (*lst)[idx] );
			}
		}
	}

	if ( mspg != 0 )  {
		_context_menu_focus.set_group_name ( mspg->group_name() );
		_context_menu_focus.set_snd_dir ( mspg->snd_dir() );
		_context_menu.setTitle ( _context_menu_focus.group_name() );

		context_menu_update();

		if ( _actions_visible > 0 ) {
			_context_menu.popup ( pos_n );
		}
	}
}


Mixer_Switches_Proxies_Group *
Mixer_Switches::context_menu_proxies_group (
	Wdg::Switches_Pad * sw_pad_n )
{
	Mixer_Switches_Proxies_Group * res ( 0 );

	if ( sw_pad_n != 0 ) {
		const int & idx ( sw_pad_n->focus_info().group_idx );
		if ( idx < sw_pad_n->proxies_groups()->size() ) {
			res = proxies_group ( (*sw_pad_n->proxies_groups())[idx] );
		}
	}

	// No focus widget found so far
	if ( ( res == 0 ) && ( !_context_menu_focus.is_clear() ) ) {

		// Find visible group by name
		for ( int ii=0; ii < _proxies_groups.size(); ++ii ) {
			Mixer_Switches_Proxies_Group * mspg ( _proxies_groups[ii] );
			if ( ( mspg->group_name() == _context_menu_focus.group_name() ) &&
				( mspg->snd_dir() == _context_menu_focus.snd_dir() ) )
			{
				res = mspg;
				break;
			}
		}
	}

	return res;
}


void
Mixer_Switches::context_menu_update ( )
{
	_actions_visible = 0;

	Wdg::Switches_Pad * sw_pad ( acquire_focus_pad() );
	Mixer_Switches_Proxies_Group * mspg ( context_menu_proxies_group ( sw_pad ) );
	if ( mspg == 0 ) {
		_context_menu.close();
		return;
	}

	const Snd_Mixer_Simple_Elem * qsme ( mspg->mixer_simple_elem() );

	// Split/Join and level channels
	{
		const bool vis_separate (
			qsme->num_channels ( mspg->snd_dir() ) > 1 );

		_act_separate_join.setVisible ( vis_separate );
		if ( vis_separate ) {
			++_actions_visible;
			if ( mspg->proxies().size() == 1 ) {
				_act_separate_join.setText ( _act_str_separate_join[0] );
			} else {
				_act_separate_join.setText ( _act_str_separate_join[1] );
			}
		}
	}

	if ( _actions_visible == 0 ) {
		_context_menu.close();
	}
}


bool
Mixer_Switches::eventFilter (
	QObject * watched,
	QEvent * event )
{
	bool filtered ( false );

	Wdg::Switches_Pad * sw_pad (
		dynamic_cast < Wdg::Switches_Pad * > ( watched ) );

	if ( sw_pad != 0 ) {
		if ( event->type() == QEvent::KeyPress ) {
			QKeyEvent * kev ( dynamic_cast < QKeyEvent * > ( event ) );
			if ( kev != 0 ) {
				filtered = true;
				QKeySequence key_seq ( kev->key() );
				if ( _act_separate_join.shortcut() == key_seq ) {
					_act_separate_join.trigger();
				} else {
					filtered = false;
				}
			}
		} else if ( event->type() == QEvent::ContextMenu ) {
			//std::cout << "Mixer_Switches::eventFilter: QContextMenuEvent\n";
			QContextMenuEvent * cmev (
				dynamic_cast < QContextMenuEvent * > ( event ) );
			if ( cmev != 0  ) {
				context_menu_start ( sw_pad, cmev->globalPos() );
				filtered = true;
			}
		}
	}

	return filtered;
}



} // End of namespace

