//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "qslider.hpp"

#include <QPainter>
#include <QMouseEvent>

#include <cassert>
#include <climits>
#include <cmath>

#include <iostream>


namespace Wdg
{


QSlider::QSlider (
	QWidget * parent,
	const QSlider_Pixmaps_Buffers & buff_n ) :
QAbstractSlider ( parent ),
_pixmaps_bg ( 0 ),
_pixmaps_handle ( 0 ),
_handle_min_height ( 15 ),
_handle_max_height ( 99 ),
_handle_max_pos ( 0 ),
_handle_pos_value ( 0 ),
_handle_pos_hint ( 0 ),
_map_hr_val ( 0 ),
_map_val_hr ( 0 ),
_update_pixmaps ( true ),
_cursor_select_allowed ( false ),
_cursor_over_handle ( false ),
_cursor_value_hinting ( false ),
_anim_run_snap ( false ),
_pixmaps_buffers ( buff_n )
{
	setSizePolicy ( QSizePolicy::Expanding, QSizePolicy::Expanding );
	setFocusPolicy ( Qt::StrongFocus );
	setMouseTracking ( true );

	// Setup the animation timer
	const int fps ( 60 );
	_anim_timer.setInterval ( 1000 / fps );
	connect ( &_anim_timer, SIGNAL ( timeout() ),
		this, SLOT ( anim_tick() ) );

	update_painter_utils();
}


QSlider::~QSlider ( )
{
	set_pixmaps_buffers ( QSlider_Pixmaps_Buffers() );
	clear_range_mappers();
}


QSize
QSlider::sizeHint ( ) const
{
	const int def_len ( fontMetrics().height() );
	return QSize ( def_len, def_len*10 );
}


QSize
QSlider::minimumSizeHint ( ) const
{
	const int def_len ( fontMetrics().height() );
	return QSize ( def_len, def_len*4 );
}


void
QSlider::set_pixmaps_config (
	const QSlider_Pixmaps_Config & cfg_n )
{
	_pixmaps_config = cfg_n;
	_update_pixmaps = true;
	update_painter_utils();
	update();
}


void
QSlider::set_pixmaps_buffers (
	const QSlider_Pixmaps_Buffers & buff_n )
{
	if ( pixmaps_buffers().buff_bg() != 0 ) {
		pixmaps_buffers().buff_bg()->return_pixmaps ( _pixmaps_bg );
	}
	if ( pixmaps_buffers().buff_handle() != 0 ) {
		pixmaps_buffers().buff_handle()->return_pixmaps ( _pixmaps_handle );
	}

	_pixmaps_buffers = buff_n;

	if ( ( pixmaps_buffers().buff_bg() != 0 ) ||
		( pixmaps_buffers().buff_handle() != 0 ) )
	{
		_update_pixmaps = true;
	}

	update_painter_utils ( );
	update();
}


void
QSlider::sliderChange (
	SliderChange change )
{
	switch ( change ) {
		case QAbstractSlider::SliderRangeChange:
			slider_change_range();
			break;
		case QAbstractSlider::SliderOrientationChange:
			break;
		case QAbstractSlider::SliderStepsChange:
			break;
		case QAbstractSlider::SliderValueChange:
			slider_change_value();
			break;
		default:
			break;
	}
	QAbstractSlider::sliderChange ( change );
	update();
}


void
QSlider::slider_change_range ( )
{
	update_range_mappers();

	unsigned int delta = integer_distance ( minimum(), maximum() );
	int div;
	int val;
	int mod;

	// Update pixmap config
	_pixmaps_config.bg.ticks_max_idx = delta;

	// Single step
	div = 100;
	val = delta / div;
	mod = delta % div;
	if ( mod > ( div / 2 ) ) {
		++val;
	}
	setSingleStep ( qMax ( val, 1 ) );

	// Page step
	div = 12;
	val = delta / div;
	mod = delta % div;
	if ( mod > ( div / 2 ) ) {
		++val;
	}
	setPageStep ( qMax ( val, 1 ) );

	_update_pixmaps = true;
}


void
QSlider::slider_change_value ( )
{
	_handle_pos_value = map_value_to_handle_pos ( value() );
	if ( !isSliderDown() ) {
		anim_snap_start();
	}
}


int
QSlider::map_handle_pos_to_value (
	int handle_pos_n )
{
	int res ( minimum() );

	if ( _handle_max_pos > 0 ) {
		int hpos = handle_pos_n;
		qBound ( 0, hpos, _handle_max_pos );
		hpos = _handle_max_pos - hpos;

		res += _map_hr_val->map ( hpos );

		assert ( res >= minimum() );
		assert ( res <= maximum() );
	}

	return res;
}


int
QSlider::map_value_to_handle_pos (
	int value_n )
{
	int res ( 0 );

	if ( _handle_max_pos > 0 ) {
		value_n = qBound ( minimum(), value_n, maximum() );

		unsigned int val ( integer_distance ( minimum(), value_n ) );

		res = _handle_max_pos - _map_val_hr->map ( val );
		assert ( res >= 0 );
		assert ( res <= _handle_max_pos );
	}

	return res;
}


int
QSlider::map_tick_pos_to_handle_pos (
	int tick_pos_n )
{
	int res = tick_pos_n;
	res -= _handle_rect.height() / 2;
	res = qBound ( 0, res, _handle_max_pos );
	return res;
}


int
QSlider::map_handle_pos_to_tick_pos (
	int handle_pos_n )
{
	int res ( handle_pos_n );
	res = qBound ( 0, res, _handle_max_pos );
	res += _handle_rect.height() / 2;
	return res;
}


int
QSlider::map_value_to_tick_pos (
	int value_n )
{
	int res = map_value_to_handle_pos ( value_n );
	res = map_handle_pos_to_tick_pos ( res );
	return res;
}


int
QSlider::map_tick_pos_to_value (
	int tick_pos_n )
{
	int res = map_tick_pos_to_handle_pos ( tick_pos_n );
	res = map_handle_pos_to_value ( res );
	return res;
}


void
QSlider::clear_range_mappers ( )
{
	if ( _map_val_hr != 0 ) {
		delete _map_val_hr;
		_map_val_hr = 0;
	}
	if ( _map_hr_val != 0 ) {
		delete _map_hr_val;
		_map_hr_val = 0;
	}
}


void
QSlider::update_range_mappers ( )
{
	typedef unsigned long ulong;

	clear_range_mappers();

	ulong handle_max_idx ( qMax ( 0, _handle_max_pos ) );
	ulong value_max_idx ( integer_distance ( minimum(), maximum() ) );

	if ( value_max_idx > handle_max_idx ) {
		_map_val_hr = new Range_Mapper_Down ( value_max_idx, handle_max_idx );
		_map_hr_val = new Range_Mapper_Up ( handle_max_idx, value_max_idx );
	} else {
		_map_val_hr = new Range_Mapper_Up ( value_max_idx, handle_max_idx );
		_map_hr_val = new Range_Mapper_Down ( handle_max_idx, value_max_idx );
	}
}


bool
QSlider::update_cursor_mode ( )
{
	//std::cout << "QSlider::update_cursor_mode " << "\n";

	bool res ( false ); // Indicates a cursor mode change

	const QPoint cur_pos ( mapFromGlobal ( QCursor::pos() ) );

	{ // Check if the cursor is over the handle
		const bool over_handle ( _handle_rect.contains ( cur_pos ) );
		if ( over_handle != _cursor_over_handle ) {
			_cursor_over_handle = over_handle;
			res = true;
		}
	}

	{ // Update cursor value hinting
		bool show_hint =
			_cursor_select_allowed && rect().contains ( cur_pos ) &&
			!_cursor_over_handle && !isSliderDown();

		if ( show_hint ) {
			int pos = cur_pos.y();
			pos = map_tick_pos_to_value ( pos );
			pos = map_value_to_handle_pos ( pos );

			if ( _handle_pos_hint != pos ) {
				_handle_pos_hint = pos;
				res = true;
			}
		}

		if ( _cursor_value_hinting != show_hint ) {
			_cursor_value_hinting = show_hint;
			res = true;
		}
	}

	{ // Update cursor shape
		Qt::CursorShape shape ( Qt::ArrowCursor );
		if ( isSliderDown() ) {
			shape = Qt::ClosedHandCursor;
		} else if ( _cursor_over_handle ) {
			shape = Qt::OpenHandCursor;
		} else if ( _cursor_value_hinting ) {
			shape = Qt::CrossCursor;
		}

		QCursor curs ( cursor() );
		if ( curs.shape() != shape ) {
			res = true;
			curs.setShape ( shape );
			setCursor ( curs );
		}
	}

	return res;
}


bool
QSlider::set_handle_pos (
	int pos_n )
{
	pos_n = qBound ( 0, pos_n, _handle_max_pos );

	const bool res ( handle_pos() != pos_n );
	if ( res ) {
		_handle_rect.moveTop ( pos_n );
	}

	return res;
}


void
QSlider::update_value_from_handle_pos ( )
{
	int val = map_handle_pos_to_value ( handle_pos() );
	if ( val != value() ) {
		setValue ( val );
	}
}


void
QSlider::finish_handle_manipulation ( )
{
	setSliderDown ( false );

	_handle_pos_value = map_value_to_handle_pos ( value() );
	anim_snap_start();

	update_cursor_mode();
}


// Focus events
void
QSlider::focusInEvent (
	QFocusEvent * event )
{
	//std::cout << "QSlider::focusInEvent " << "\n";

	switch ( event->reason() ) {
		case Qt::MouseFocusReason:
		case Qt::ActiveWindowFocusReason:
			_cursor_select_allowed = false;
			break;

		default:
			_cursor_select_allowed = true;
			break;
	}
	update_cursor_mode();
	update();
}


void
QSlider::focusOutEvent (
	QFocusEvent * )
{
	//std::cout << "QSlider::focusOutEvent " << "\n";

	_cursor_select_allowed = false;
	_cursor_value_hinting = false;
	finish_handle_manipulation();
	update();
}


void
QSlider::enterEvent (
	QEvent * )
{
	//std::cout << "QSlider::enterEvent" << "\n";
	update_cursor_mode();
	update();
}


void
QSlider::leaveEvent (
	QEvent * )
{
	//std::cout << "QSlider::leaveEvent" << "\n";
	update_cursor_mode();
	update();
}


// Mouse events
void
QSlider::mousePressEvent (
	QMouseEvent * event_n )
{
	//std::cout << "QSlider::mousePressEvent" << "\n";

	if ( event_n->button() == Qt::LeftButton ) {

		if ( _cursor_over_handle ) {
			// Start slider drag
			setSliderDown ( true );
		} else {
			// Select a value on the slider range
			if ( _cursor_select_allowed ) {
				setSliderDown ( true );
				const int pos (
					map_tick_pos_to_handle_pos ( event_n->pos().y() ) );
				if ( set_handle_pos ( pos ) ) {
					update_value_from_handle_pos();
				}
			}
		}

		if ( isSliderDown() ) {
			anim_stop();
		}

		// After a click selection should be allowed
		if ( hasFocus() ) {
			_cursor_select_allowed = true;
		}
		update_cursor_mode();
		update();
	}

}


void
QSlider::mouseReleaseEvent (
	QMouseEvent * event_n )
{
	//std::cout << "QSlider::mouseReleaseEvent" << "\n";

	if ( event_n->button() == Qt::LeftButton ) {
		if ( isSliderDown() ) {
			finish_handle_manipulation();
			update();
		}
	}
}


void
QSlider::mouseMoveEvent (
	QMouseEvent * event_n )
{
	//std::cout << "QSlider::mouseMoveEvent" << "\n";

	bool do_update ( false );

	if ( isSliderDown() ) {
		int delta ( 0 );

		{
			int p_now ( event_n->pos().y() );
			int p_prev ( _mouse_last.y() );

			if ( p_now != p_prev ) {
				int p_min;
				int p_max;
				if ( p_now > p_prev ) {
					// For moving downwards the cursor positions must be in this range
					p_min = _tick_min;
					p_max = height() - 1;
				} else {
					// For moving upwards the cursor positions must be in this range
					p_min = 0;
					p_max = _tick_max;
				}
				p_now = qBound ( p_min, p_now, p_max );
				p_prev = qBound ( p_min, p_prev, p_max );

				delta = p_now - p_prev;
			}
		}

		//std::cout << "QSlider::mouseMoveEvent: Delta " << delta << "\n";
		if ( set_handle_pos ( handle_pos() + delta ) ) {
			update_value_from_handle_pos();
			do_update = true;
		}
	}

	_mouse_last = event_n->pos();

	// Test if the cursor is over the handle
	if ( update_cursor_mode() ) {
		do_update = true;
	}

	if ( do_update ) {
		update();
	}
}


void
QSlider::wheelEvent (
	QWheelEvent * event )
{
	if ( rect().contains ( event->pos() ) ) {
		event->accept();
		setFocus();
		unsigned int val_delta = integer_distance ( minimum(), maximum() );
		int add = ( event->delta() / 8 ) * ( val_delta / 225 );
		if ( add == 0 ) {
			if ( event->delta() > 0 ) {
				add = 1;
			} else {
				add = -1;
			}
		}

		//std::cout << "wheelEvent " << add << "\n";
		setValue ( value() + add );
	} else {
		event->ignore();
	}
}


// Window events
void
QSlider::resizeEvent (
	QResizeEvent * )
{
	anim_stop();

	if ( ( width() <= 0 ) || ( height() <= 0 ) ) {
		return;
	}

	// Handle rect
	int hh;
	hh = height() / 10;
	hh = qBound ( _handle_min_height, hh, _handle_max_height );
	hh += ( hh + 1 ) % 2; // Make odd

	_handle_rect.setRect ( 0, 0, width(), hh );

	// Handle range
	_handle_max_pos = height() - _handle_rect.height();
	update_range_mappers();

	_tick_min = _handle_rect.height() / 2;
	_tick_max = _tick_min + ( _handle_max_pos + 1 );

	// Update pixmap config
	_pixmaps_config.bg.ticks_range_max_idx = qMax ( 0, _handle_max_pos );
	_pixmaps_config.bg.ticks_range_start = _handle_rect.height() / 2;


	// Update handle position
	_handle_pos_value = map_value_to_handle_pos ( value() );
	set_handle_pos ( _handle_pos_value );
	update_cursor_mode();

	// Request new pixmaps
	_update_pixmaps = true;
}


void
QSlider::paintEvent (
	QPaintEvent * )
{
	if ( _update_pixmaps ) {
		_update_pixmaps = false;
		update_pixmaps();
	}

	QPainter painter ( this );

	// Draw slider background
	if ( _pixmaps_bg != 0 ) {
		int idx ( 0 );
		if ( hasFocus() ) {
			idx = 1;
		} else {
			if ( underMouse() ) {
				idx = 2;
			}
		}
		painter.drawPixmap (
			QPoint ( 0, 0 ),
			_pixmaps_bg->pixmap ( idx ) );
	}

	// Draw value marker
	if ( isSliderDown() || _anim_run_snap ) {

		// Draw snapping line
		const int pdiff ( abs ( handle_pos() - _handle_pos_value ) );
		if ( pdiff > 5 ) {
			const qreal xx ( width() / 2.0 );
			const qreal dx1 ( 1.5 );
			qreal yy1 ( map_handle_pos_to_tick_pos ( handle_pos()      ) );
			qreal yy2 ( map_handle_pos_to_tick_pos ( _handle_pos_value ) );
			yy1 += 0.5;
			yy2 += 0.5;
			const qreal yym ( ( yy1 + yy2 ) / 2.0 );

			const int num_pts ( 6 );
			QPointF pts[num_pts] = {
				QPointF ( xx + dx1, yy1 ),
				QPointF ( xx      , yym ),
				QPointF ( xx + dx1, yy2 ),
				QPointF ( xx - dx1, yy2 ),
				QPointF ( xx      , yym ),
				QPointF ( xx - dx1, yy1 )
			};
			painter.setRenderHint ( QPainter::Antialiasing );
			painter.setPen ( _snap_pen );
			painter.setBrush ( _snap_brush );
			painter.drawPolygon ( pts, num_pts );
		}

		// Draw value marker
		{
			QPointF pp (
				_marker_offset[0],
				_marker_offset[1] + _handle_pos_value );
			painter.drawPixmap (
				pp, _pixmaps_handle->pixmap ( 5 ) );
		}
	}

	// Draw value hint marker
	if ( _cursor_value_hinting ) {
		QPointF pp (
			_marker_offset[0],
			_marker_offset[1] + _handle_pos_hint );
		painter.drawPixmap (
			pp, _pixmaps_handle->pixmap ( 6 ) );
	}

	// Draw handle
	if ( _pixmaps_handle != 0 ) {
		int idx;
		if ( isSliderDown() ) {
			idx = 4;
		} else {
			idx = hasFocus() ? 1 : 0;
			if ( underMouse() ) {
				idx += 2;
			}
		}
		painter.drawPixmap (
			_handle_rect.topLeft(),
			_pixmaps_handle->pixmap ( idx ) );
	}
}


void
QSlider::update_pixmaps ( )
{
	_pixmaps_config.bg.size = size();
	_pixmaps_config.handle.size = _handle_rect.size();

	// Acquire background pixmaps
	if ( pixmaps_buffers().buff_bg() != 0 ) {
		_pixmaps_bg = pixmaps_buffers().buff_bg()->return_acquire (
			_pixmaps_bg, _pixmaps_config.bg );
	}

	// Acquire handle pixmaps
	if ( pixmaps_buffers().buff_handle() != 0 ) {
		_pixmaps_handle = pixmaps_buffers().buff_handle()->return_acquire (
			_pixmaps_handle, _pixmaps_config.handle );
	}

	// Update value marker painting position offset
	{
		int off_x ( width() );
		int off_y ( _handle_rect.height() / 2 );
		if ( _pixmaps_handle != 0 ) {
			QSize ps ( _pixmaps_handle->pixmap ( 5 ).size() );
			off_x -= ps.width();
			off_y -= ps.height() / 2;
		}
		off_x /= 2;
		_marker_offset[0] = off_x;
		_marker_offset[1] = off_y;
	}

	// Update handle position
	_handle_pos_value = map_value_to_handle_pos ( value() );
	set_handle_pos ( _handle_pos_value );
	update_cursor_mode();
}


void
QSlider::update_painter_utils ( )
{
	QColor col_fg ( palette().color ( QPalette::WindowText ) );
	if ( pixmaps_buffers().painter() != 0 ) {
		const int style_id ( pixmaps_config().bg.style_id );
		const QWidget_Style * style (
			pixmaps_buffers().painter()->style ( style_id ) );
		if ( style != 0 ) {
			col_fg = style->col_fg;
		}
	}

	col_fg.setAlpha ( 90 );

	_snap_pen.setStyle ( Qt::NoPen );
	_snap_pen.setColor ( col_fg );

	_snap_brush.setStyle ( Qt::SolidPattern );
	_snap_brush.setColor ( col_fg );
}


void
QSlider::anim_stop ( )
{
	_anim_run_snap = false;
	_anim_timer.stop();
}


void
QSlider::anim_tick ( )
{
	bool end_reached ( false );
	bool do_update ( false );

	// Passed seconds since the animation start
	int msec ( abs ( _anim_time.elapsed() ) );
	if ( msec >= _anim_msec_max ) {
		msec = _anim_msec_max;
		end_reached = true;
	}

	if ( _anim_run_snap ) {
		if ( anim_snap_tick ( msec ) ) {
			do_update = true;
		}
	}

	if ( end_reached ) {
		anim_stop();
	}
	if ( do_update ) {
		update_cursor_mode();
		update();
	}
}


void
QSlider::anim_snap_start ( )
{
	bool do_snap ( handle_pos() != _handle_pos_value );
	if ( do_snap ) {

		double v_x0 ( 0.0 );

		if ( _anim_run_snap ) {
			// A snapping annimation is already running.
			// Acquire the current speed and give it a tick
			int msec ( abs ( _anim_time.elapsed() ) );
			if ( msec < _anim_msec_max ) {
				v_x0 = _cubic_curve.eval_speed ( double ( msec ) / 1000.0 );
			} else {
				msec = _anim_msec_max;
			}
			anim_snap_tick ( msec );
		}

		_anim_time.start();
		if ( !_anim_timer.isActive() ) {
			_anim_timer.start();
		}
		_anim_run_snap = true;

		const int xdiff ( _handle_pos_value - handle_pos() );
		_anim_msec_max = ( 800 * abs ( xdiff ) ) / _handle_max_pos;
		_anim_msec_max = qBound ( 100, _anim_msec_max, 600 );

		_cubic_curve.set_params (
			handle_pos(), v_x0,
			_handle_pos_value,
			_anim_msec_max / 1000.0 );
	}
}


bool
QSlider::anim_snap_tick (
	int msec_n )
{
	bool do_update ( false );

	int hpos ( _handle_pos_value );
	if ( msec_n <= _anim_msec_max ) {
		// Calculate the position from the easing curve
		double dpos ( double ( msec_n ) / 1000.0 );
		dpos = _cubic_curve.eval ( dpos );
		dpos = std::floor ( dpos + 0.5 ); // Round
		hpos = static_cast < int > ( dpos );
	}

	if ( set_handle_pos ( static_cast < int > ( hpos ) ) ) {
		do_update = true;
	}

	return do_update;
}


unsigned int
QSlider::integer_distance (
	int min_val,
	int max_val )
{
	unsigned int res ( 0 );

	if ( min_val > max_val ) {
		int tmp = max_val;
		max_val = min_val;
		min_val = tmp;
	}

	if ( ( min_val >= 0 ) || ( max_val < 0 ) ) {
		// No integer overflow possible
		res = int ( max_val - min_val );
	} else {
		// Integer overflow possible
		res = max_val;
		++min_val;
		res += long ( -min_val );
		++res;
	}

	return res;
}


} // End of namespace

