//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_qslider_hpp__
#define __INC_qslider_hpp__

#include <QAbstractSlider>
#include <QTime>
#include <QTimer>
#include <QPen>
#include <QBrush>

#include "qslider_pixmaps_buffers.hpp"
#include "range_mapper.hpp"
#include "cubic_curve.hpp"


namespace Wdg
{


class QSlider :
	public QAbstractSlider
{
	Q_OBJECT

	// Public methods
	public:

	QSlider (
		QWidget * parent = 0,
		const QSlider_Pixmaps_Buffers & buff_n = QSlider_Pixmaps_Buffers() );

	~QSlider ( );

	QSize
	sizeHint ( ) const;

	QSize
	minimumSizeHint ( ) const;



	// Buffers

	const QSlider_Pixmaps_Buffers &
	pixmaps_buffers ( ) const;

	void
	set_pixmaps_buffers (
		const QSlider_Pixmaps_Buffers & buff_n );


	// Pixmap config

	const QSlider_Pixmaps_Config &
	pixmaps_config ( ) const;

	void
	set_pixmaps_config (
		const QSlider_Pixmaps_Config & cfg_n );


	// Utility

	static
	unsigned int
	integer_distance (
		int min_val,
		int max_val );


	// Protected methods
	protected:

	// Slider change callbacks

	void
	sliderChange (
		SliderChange change );

	void
	slider_change_range ( );

	void
	slider_change_value ( );



	// Mapping: Pixel <=> Value

	int
	map_handle_pos_to_value (
		int handle_pos_n );

	int
	map_value_to_handle_pos (
		int value_n );

	int
	map_tick_pos_to_handle_pos (
		int tick_pos_n );

	int
	map_handle_pos_to_tick_pos (
		int handle_pos_n );

	int
	map_tick_pos_to_value (
		int tick_pos_n );

	int
	map_value_to_tick_pos (
		int value_n );

	void
	clear_range_mappers ( );

	void
	update_range_mappers ( );


	// Cursor mode

	bool
	update_cursor_mode ( );


	// Handle position

	int
	handle_pos ( );

	/// @return true if the handle position was changed
	bool
	set_handle_pos (
		int pos_n );


	void
	update_value_from_handle_pos ( );

	void
	finish_handle_manipulation ( );


	// Focus events
	void
	focusInEvent (
		QFocusEvent * event );

	void
	focusOutEvent (
		QFocusEvent * event );


	// Enter events
	void
	enterEvent (
		QEvent * event );

	void
	leaveEvent (
		QEvent * event );


	// Mouse events
	void
	mousePressEvent (
		QMouseEvent * event );

	void
	mouseReleaseEvent (
		QMouseEvent * event );

	void
	mouseMoveEvent (
		QMouseEvent * event );

	void
	wheelEvent (
		QWheelEvent * e );


	// Window events
	void
	resizeEvent (
		QResizeEvent * event );

	void
	paintEvent (
		QPaintEvent * event );


	/// @brief Fetches new pixmaps from the image buffers
	void
	update_pixmaps ( );

	void
	update_painter_utils ( );



	// Private slots
	private slots:

	void
	anim_tick ( );


	// Private methods
	private:

	void
	anim_stop ( );

	void
	anim_snap_start ( );

	bool
	anim_snap_tick (
		int msec_n );



	// Private attributes
	private:

	// Pixmap sets
	SPix::Shared_Pixmap_Set * _pixmaps_bg;
	SPix::Shared_Pixmap_Set * _pixmaps_handle;

	QRect _handle_rect;
	QPen _snap_pen;
	QBrush _snap_brush;

	const int _handle_min_height;
	const int _handle_max_height;
	int _handle_max_pos;
	int _handle_pos_value;
	int _handle_pos_hint;
	int _marker_offset[2];
	int _tick_min;
	int _tick_max;
	QPoint _mouse_last;

	Range_Mapper * _map_hr_val;
	Range_Mapper * _map_val_hr;

	bool _update_pixmaps;
	bool _cursor_select_allowed;
	bool _cursor_over_handle;
	bool _cursor_value_hinting;
	bool _anim_run_snap;

	// Pixmap buffers
	QSlider_Pixmaps_Config _pixmaps_config;
	QSlider_Pixmaps_Buffers _pixmaps_buffers;

	// Animation
	QTimer _anim_timer;
	QTime _anim_time;
	int _anim_msec_max;

	Cubic_Curve _cubic_curve;
};


inline
const QSlider_Pixmaps_Buffers &
QSlider::pixmaps_buffers ( ) const
{
	return _pixmaps_buffers;
}


inline
const QSlider_Pixmaps_Config &
QSlider::pixmaps_config ( ) const
{
	return _pixmaps_config;
}


inline
int
QSlider::handle_pos ( )
{
	return _handle_rect.top();
}


} // End of namespace

#endif


