//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "sliders_pad_labels.hpp"

#include <QMouseEvent>
#include <QPainter>

#define _USE_MATH_DEFINES
#include <cmath>
#include <climits>

#include <iostream>


namespace Wdg
{


//
// Sliders_Pad_Labels
//

Sliders_Pad_Labels::Sliders_Pad_Labels (
	Sliders_Pad_Widgets_Group_List & widgets_n,
	Sliders_Pad_Style & style_n,
	QWidget * parent_n ) :
QWidget ( parent_n ),
_widgets_groups ( widgets_n ),
_sp_style ( style_n ),
_weak_focus_idx ( -1 )
{
	_vars.max_str_length_px = fontMetrics().averageCharWidth() * 18;

	setMouseTracking ( true );

	int max_h ( _vars.spacing_bottom );
	max_h += _vars.pad_left;
	max_h += _vars.pad_right;
	max_h += _vars.max_str_length_px;
	setMaximumHeight ( max_h  );
}


QSize
Sliders_Pad_Labels::minimumSizeHint ( ) const
{
	QSize res ( 0, 0 );
	res.setWidth ( 64 );
	res.setHeight ( _vars.spacing_bottom + fontMetrics().height() * 2 );

	return res;
}


QSize
Sliders_Pad_Labels::sizeHint ( ) const
{
	return minimumSizeHint();
}


void
Sliders_Pad_Labels::update_widgets_graphics ( )
{
	update_labels_transforms();
	update_stem_graphics();
}


void
Sliders_Pad_Labels::update_labels_transforms ( )
{
	const qreal fnt_h ( fontMetrics().height() );

	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );

		qreal x_center ( spwg->group_pos() );
		x_center += spwg->group_width() / 2.0;
		x_center += _vars.center_x;

		// Transformation
		{
			QTransform trans (
				_vars.angle_cos, -_vars.angle_sin, 0.0,
				_vars.angle_sin, _vars.angle_cos, 0.0,
				x_center, height() - _vars.spacing_bottom, 1.0 );
			spwg->set_label_transform ( trans );
		}


		// Rectangle

		qreal lbl_width;
		if ( _vars.angle_sin > 0.002 ) {
			qreal hh = height();
			hh -= _vars.spacing_bottom;
			hh -= fnt_h * _vars.angle_cos;
			lbl_width = hh / _vars.angle_sin;
		} else {
			lbl_width = _vars.max_str_length_px * 2;
		}

		if ( _vars.angle_cos > 0.002 ) {
			// Shorten the length if the label is too near to the right edge
			qreal xdist ( ( width() - x_center ) / _vars.angle_cos );
			if ( xdist < lbl_width ) {
				lbl_width = xdist;
			}
		}

		const QRectF lbl_rect ( 0, -fnt_h, lbl_width, fnt_h );
		spwg->set_label_rect ( lbl_rect );
	}
}


void
Sliders_Pad_Labels::update_stem_graphics ( )
{
	const qreal fnt_hhalf ( fontMetrics().height() / 2.0 );
	const qreal c_off ( _sp_style.stem_corner_indent );
	const qreal y_bottom ( height() );
	const qreal y_mid ( ( height() - _vars.spacing_inter ) + qreal ( 0.5 ) );

	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
		QPainterPath & ppath ( spwg->ppath_top() );
		ppath = QPainterPath();

		if ( spwg->num_sliders() == 0 ) {
			continue;
		}

		const unsigned int sl_max_idx ( spwg->num_sliders() - 1 );
		qreal x_mid;
		qreal stem_height;
		qreal stem_bottom;
		qreal stem_x1 ( 0.0 );
		qreal stem_h1 ( 0.0 );
		qreal stem_h2 ( 0.0 );


		x_mid = spwg->group_width() / qreal ( 2.0 );

		stem_height = _vars.angle_cos * fnt_hhalf;
		stem_bottom = y_bottom;
		if ( spwg->num_sliders() == 1 ) {
			stem_height += _vars.spacing_bottom;
		} else {
			stem_height += 0.5;
			stem_bottom = y_mid;
		}

		if ( stem_height > 0.5 ) {
			qreal ll = stem_height / ( 1.0 + _vars.angle_sin );
			stem_x1 = _vars.angle_cos * ll;
			stem_x1 = std::floor ( stem_x1 );
			if ( ( spwg->group_width() % 2 ) == 0 ) {
				stem_x1 += 0.5;
			}

			if ( _vars.angle_cos > 0.002 ) {
				stem_h2 = stem_x1 * _vars.angle_sin / _vars.angle_cos;
			} else {
				stem_h2 = 0.0;
			}

			stem_x1 = x_mid - stem_x1;
			stem_h1 = stem_height - stem_h2;

			ppath.moveTo ( x_mid, stem_bottom - stem_height );
			ppath.lineTo ( stem_x1, stem_bottom - stem_h1 );
			ppath.lineTo ( stem_x1, stem_bottom );
		}

		if ( spwg->num_sliders() == 1 ) {
			continue;
		}

		// Prepare bridge lines

		// Draw bridge line from the leftmost slider to the rightmost
		{
			const Sliders_Pad_Widgets * spw_b ( spwg->widgets ( 0 ) );
			const Sliders_Pad_Widgets * spw_e ( spwg->widgets ( sl_max_idx ) );

			qreal cc_min ( spw_b->pos() - spwg->group_pos() );
			qreal cc_max ( spw_e->pos() - spwg->group_pos() );

			cc_min += _sp_style.calc_center (
				spw_b->width(), 0, spwg->num_sliders() );

			cc_max += _sp_style.calc_center (
				spw_b->width(), sl_max_idx, spwg->num_sliders() );

			ppath.moveTo ( cc_min, y_bottom );
			ppath.lineTo ( cc_min, y_mid + c_off );
			ppath.lineTo ( cc_min + c_off, y_mid );
			ppath.lineTo ( cc_max - c_off, y_mid );
			ppath.lineTo ( cc_max, y_mid + c_off );
			ppath.lineTo ( cc_max, y_bottom );
		}

		// Draw vertical lines from the bridge line down to the inner sliders
		for ( unsigned int jj=1; jj < sl_max_idx; ++jj ) {
			const Sliders_Pad_Widgets * spw ( spwg->widgets ( jj ) );

			qreal cc ( spw->pos() - spwg->group_pos() );
			cc += _sp_style.calc_center (
				spw->width(), jj, spwg->num_sliders() );

			ppath.moveTo ( cc, y_mid );
			ppath.lineTo ( cc, y_bottom );
		}

		// Prepare focus lines for each slider
		for ( unsigned int jj=0; jj < spwg->num_sliders(); ++jj ) {
			Sliders_Pad_Widgets * spw ( spwg->widgets ( jj ) );
			QPainterPath & pp ( spw->ppath_top_focus() );
			pp = QPainterPath();

			pp.moveTo ( x_mid, stem_bottom - stem_height );
			pp.lineTo ( stem_x1, stem_bottom - stem_h1 );
			pp.lineTo ( stem_x1, y_mid );

			qreal cc ( spw->pos() - spwg->group_pos() );
			cc += _sp_style.calc_center (
				spw->width(), jj, spwg->num_sliders() );

			if ( jj == 0 ) {
				pp.lineTo ( cc + c_off, y_mid );
				pp.lineTo ( cc, y_mid + c_off );
				pp.lineTo ( cc, y_bottom );

			} else if ( jj == sl_max_idx ) {

				pp.lineTo ( cc - c_off, y_mid );
				pp.lineTo ( cc, y_mid + c_off );
				pp.lineTo ( cc, y_bottom );

			} else {
				pp.lineTo ( cc, y_mid );
				pp.lineTo ( cc, y_bottom );
			}
		}
	}
}


void
Sliders_Pad_Labels::enterEvent (
	QEvent * )
{
	if ( _weak_focus_idx >= 0 ) {
		_weak_focus_idx = -1;
		update();
	}
}


void
Sliders_Pad_Labels::leaveEvent (
	QEvent * )
{
	if ( _weak_focus_idx >= 0 ) {
		_weak_focus_idx = -1;
		update();
	}
}


void
Sliders_Pad_Labels::mousePressEvent (
	QMouseEvent * event_n )
{
	const int idx ( find_label ( event_n->pos() ) );
	if ( idx >= 0 ) {
		set_group_focus ( idx );

		QCursor cursor_new ( cursor() );
		cursor_new.setShape ( Qt::ArrowCursor );
		setCursor ( cursor_new );
	}
}


void
Sliders_Pad_Labels::mouseMoveEvent (
	QMouseEvent * event_n )
{
	const int idx ( find_label ( event_n->pos() ) );
	if ( idx != _weak_focus_idx ) {
		_weak_focus_idx = idx;

		Qt::CursorShape cursor_shape ( Qt::ArrowCursor );

		if ( idx < 0 ) {
			setToolTip ( QString() );
		} else {
			setToolTip ( _widgets_groups[idx]->proxies_group()->tool_tip() );
			if ( ( event_n->buttons() & Qt::LeftButton ) != 0 ) {
				set_group_focus ( idx );
			} else {
				cursor_shape = Qt::PointingHandCursor;
			}
		}

		{
			QCursor cursor_new ( cursor() );
			if ( cursor_new.shape() != cursor_shape ) {
				cursor_new.setShape ( cursor_shape );
				setCursor ( cursor_new );
			}
		}
		update();
	}
}


void
Sliders_Pad_Labels::paintEvent (
	QPaintEvent * event_n )
{
	//std::cout << "Sliders_Pad_Labels::paintEvent" << "\n";

	QWidget::paintEvent ( event_n );
	if ( _widgets_groups.size() == 0 ) {
		return;
	}

	QPainter painter ( this );
	QPen pen_text;

	QBrush brush_label_focus;
	brush_label_focus.setColor ( palette().color ( QPalette::Highlight ) );
	brush_label_focus.setStyle ( Qt::SolidPattern );

	QFont fnt_weak_focus ( font() );
	fnt_weak_focus.setUnderline ( !fnt_weak_focus.underline() );


	//QBrush brush_text;

	//painter.drawRect ( rect() );
	painter.setRenderHints (
		QPainter::Antialiasing |
		QPainter::TextAntialiasing |
		QPainter::SmoothPixmapTransform );


	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		const Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );

		if ( spwg->num_sliders() == 0 ) {
			continue;
		}

		bool has_slider_focus ( false );
		Sliders_Pad_Widgets * spw_foc ( spwg->focus_widgets() );
		if ( spw_foc != 0 ) {
			if ( spw_foc->slider()->hasFocus() ) {
				has_slider_focus = true;
			}
		}
		//std::cout << "Has focus " << has_slider_focus << "\n";

		if ( has_slider_focus ) {
			pen_text.setColor ( palette().color ( QPalette::HighlightedText ) );
		} else {
			//pen_text.setColor ( palette().color ( QPalette::WindowText ) );
			pen_text.setColor ( spwg->col_text() );
		}

		//
		// Stem
		//
		{
			QTransform trans (
				1.0, 0.0, 0.0,
				0.0, 1.0, 0.0,
				spwg->group_pos(), 0.0, 1.0 );
			painter.setTransform ( trans );
		}

		painter.setBrush ( Qt::NoBrush );

		if ( has_slider_focus ) {
			if ( spwg->num_sliders() == 1 ) {
				painter.setPen ( spwg->pen_stem_focus() );
				painter.drawPath ( spwg->ppath_top() );
			} else {
				painter.setPen ( spwg->pen_stem_normal() );
				painter.drawPath ( spwg->ppath_top() );

				painter.setPen ( spwg->pen_stem_focus() );
				painter.drawPath ( spw_foc->ppath_top_focus() );
			}
		} else {
			painter.setPen ( spwg->pen_stem_normal() );
			painter.drawPath ( spwg->ppath_top() );
		}


		//
		// Inclined area
		//

		painter.setTransform ( spwg->label_transform() );
		QRectF lbl_rect ( spwg->label_rect() );

		// Draw inclined area background
		if ( has_slider_focus ) {
			painter.setPen ( Qt::NoPen );
			painter.setBrush ( brush_label_focus );
			painter.drawRect ( lbl_rect );
		}

		// Draw text
		{
			painter.setPen ( pen_text );
			painter.setBrush ( Qt::NoBrush );

			if ( ( _weak_focus_idx == ii ) && !has_slider_focus ) {
				painter.setFont ( fnt_weak_focus );
			} else {
				painter.setFont ( font() );
			}

			// Elide text on demand
			QString lbl_txt ( spwg->group_name() );
			lbl_txt = fontMetrics().elidedText (
				lbl_txt, Qt::ElideRight, lbl_rect.width() - _vars.pad_left );

			lbl_rect.setLeft ( lbl_rect.left() + _vars.pad_left );
			painter.drawText ( lbl_rect, Qt::AlignLeft | Qt::AlignTop, lbl_txt );
		}
	}

}


int
Sliders_Pad_Labels::find_label (
	QPoint pos_n )
{
	int res ( -1 );

	QPointF pos_f ( pos_n );
	for ( int ii = ( _widgets_groups.size() - 1 ); ii >= 0; --ii ) {
		const Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );

		if ( pos_n.x() >= spwg->group_pos() ) {
			QPointF pos_map = spwg->label_transform_inv().map ( pos_f );
			QRectF rect_f ( spwg->label_rect() );
			rect_f.adjust ( 0, -1.5, 0, 3.0 );
			if ( rect_f.contains ( pos_map ) ) {
				res = ii;
				break;
			}
		}
	}

	return res;
}


void
Sliders_Pad_Labels::set_group_focus (
	int group_idx_n )
{
	if ( group_idx_n < _widgets_groups.size() ) {
		QWidget	* wdg ( _widgets_groups[group_idx_n]->slider ( 0 ) );
		if ( wdg != 0 ) {
			wdg->setFocus();
		}
	}
}


} // End of namespace

