//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "sliders_pad_layout.hpp"

#define _USE_MATH_DEFINES
#include <cmath>
#include <climits>

#include <QApplication>
#include <QFontMetrics>

#include <iostream>


namespace Wdg
{


Sliders_Pad_Layout::Sliders_Pad_Layout (
	Sliders_Pad_Widgets_Group_List & groups_n,
	Sliders_Pad_Labels_Vars & labels_vars_n,
	QWidget * parent_n,
	QWidget * labels_n ) :
_widgets_groups ( groups_n ),
_parent ( parent_n ),
_labels ( labels_n ),
_labels_vars ( labels_vars_n ),
_group_min_width ( 0 ),
_min_spacing_horizontal_major ( 4 ),
_min_spacing_horizontal_minor ( 2 ),
_spacing_vertical ( 7 ),
_input_area_min_width ( 64 ),
_input_area_min_height ( 64 ),
_weight_normal ( 2048 )
{
	widgets_groups_changed();
}


void
Sliders_Pad_Layout::normalize_weights (
	unsigned long * weights_n )
{
	unsigned long weights_sum ( 0 );
	for ( int ii=0; ii < 4; ++ii ) {
		weights_sum += weights_n[ii];
	}

	unsigned long weights_cur[4];
	unsigned long weights_cur_mod[4];
	for ( int ii=0; ii < 4; ++ii ) {
		weights_cur[ii] = ( weights_n[ii] * _weight_normal );
	}
	for ( int ii=0; ii < 4; ++ii ) {
		weights_cur_mod[ii] = ( weights_cur[ii] % weights_sum );
	}
	for ( int ii=0; ii < 4; ++ii ) {
		weights_cur[ii] = ( weights_cur[ii] / weights_sum );
	}


	while ( true ) {

		unsigned long weights_cur_sum ( 0 );
		for ( int ii=0; ii < 4; ++ii ) {
			weights_cur_sum += weights_cur[ii];
		}

		if ( weights_cur_sum == _weight_normal ) {
			break;
		}

		unsigned long mod_max ( 0 );
		int mod_max_idx ( -1 );
		for ( int ii=0; ii < 4; ++ii ) {
			const unsigned long & cmod ( weights_cur_mod[ii] );
			if ( cmod > mod_max ) {
				mod_max = cmod;
				mod_max_idx = ii;
			}
		}
		if ( mod_max_idx >= 0 ) {
			weights_cur[mod_max_idx] += 1;
			weights_cur_mod[mod_max_idx] = 0;
		} else {
			weights_cur[0] += _weight_normal - weights_cur_sum;
		}

	}


	for ( unsigned int ii=0; ii < 4; ++ii ) {
		weights_n[ii] = weights_cur[ii];
	}
}


void
Sliders_Pad_Layout::calc_weights_delta (
	long * weights_delta_n,
	const unsigned long * weights_requested_norm_n,
	const unsigned long * weights_currently_n )
{
	unsigned long weights_single_cur[4];
	for ( int ii=0; ii < 4; ++ii ) {
		weights_single_cur[ii] = weights_currently_n[ii];
	}
	normalize_weights ( weights_single_cur );

	for ( int ii=0; ii < 4; ++ii ) {
		weights_delta_n[ii] = weights_requested_norm_n[ii] - weights_single_cur[ii];
		//std::cout << "Delta " << ii << ": " << weights_delta_n[ii] << "\n";
	}
}


void
Sliders_Pad_Layout::calc_weights_delta_sorted (
	unsigned int * sorting_n,
	const long * weights_deltas_n )
{
	long delta_signed[4];
	for ( int ii=0; ii < 4; ++ii ) {
		delta_signed[ii] = weights_deltas_n[ii];
	}

	for ( int ii=0; ii < 4; ++ii ) {
		long min_val ( delta_signed[0] );
		int min_idx ( 0 );
		for ( int jj=0; jj < 4; ++jj ) {
			if ( delta_signed[jj] < min_val ) {
				min_val = delta_signed[jj];
				min_idx = jj;
			}
		}
		delta_signed[min_idx] = LONG_MAX;
		sorting_n[ii] = min_idx;
		//std::cout << "Sorted " << ii << ": " << sorting_n[ii] << "\n";
	}
}



QSize
Sliders_Pad_Layout::minimumSizeHint ( ) const
{
	QSize res ( 0, 0 );
	int & ww ( res.rwidth() );
	int & hh ( res.rheight() );

	{
		int mg[4] = { 0, 0, 0, 0 };
		_parent->getContentsMargins ( &mg[0], &mg[1], &mg[2], &mg[3] );
		ww += mg[0] + mg[2];
		hh += mg[1] + mg[3];
	}

	ww += _input_area_min_width;
	hh += _input_area_min_height;

	{
		const QSize lmsh ( _labels->minimumSizeHint() );
		hh += lmsh.height();
	}

	return res;
}


QSize
Sliders_Pad_Layout::sizeHint ( ) const
{
	QSize res ( 0, 0 );
	int & ww ( res.rwidth() );
	int & hh ( res.rheight() );

	{
		int mg[4] = { 0, 0, 0, 0 };
		_parent->getContentsMargins ( &mg[0], &mg[1], &mg[2], &mg[3] );
		ww += mg[0] + mg[2];
		hh += mg[1] + mg[3];
	}

	ww += _input_area_min_width * 3 / 2;
	hh += _input_area_min_height * 3 / 2;

	{
		const QSize lmsh ( _labels->sizeHint() );
		hh += lmsh.height();
	}

	return res;
}


void
Sliders_Pad_Layout::widgets_groups_changed ( )
{
	update_type_count();
	update_size_hints();
}


void
Sliders_Pad_Layout::update_basic_sizes ( )
{
	_switch_height_min = qMax ( QApplication::fontMetrics().height(), 16 );
	_slider_width_min = _switch_height_min;
}


void
Sliders_Pad_Layout::update_type_count ( )
{
	// Update number of item types
	for ( int ii=0; ii < 4; ++ii ) {
		_type_count[ii] = 0;
	}

	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		const Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
		const int num_wdg ( spwg->widgets().size() );

		// Some channels are visible
		if ( num_wdg > 0 ) {
			_type_count[1] += 1; // Spacing major
			if ( num_wdg == 1 ) {
				// Single major slider
				_type_count[3] += 1;
			} else {
				// More than one minor slider
				_type_count[0] += ( num_wdg - 1 );
				_type_count[2] += num_wdg;
			}
		}
	}
}



void
Sliders_Pad_Layout::update_size_hints ( )
{
	update_basic_sizes();

	int cb_height_min ( 16 );
	int sl_height_min ( 16*4 );

	if ( _widgets_groups.size() > 0 ) {

		for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
			const Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
			const int num_cols_ch ( spwg->widgets().size() );
			for ( int ii=0; ii < num_cols_ch; ++ii ) {

				QWidget * wdg ( spwg->slider ( ii ) );
				if ( wdg != 0 ) {
					const QSize msh ( wdg->minimumSizeHint() );
					sl_height_min = qMax ( msh.height(), sl_height_min );
				}

				wdg = spwg->switch_wdg ( ii );
				if ( wdg != 0 ) {
					const QSize msh ( wdg->minimumSizeHint() );
					cb_height_min = qMax ( msh.height(), cb_height_min );
				}
			}
		}

	}

	int & ww ( _input_area_min_width );
	int & hh ( _input_area_min_height );

	ww = 0;
	hh = 0;

	ww += _type_count[0] * _min_spacing_horizontal_minor;
	ww += _type_count[1] * _min_spacing_horizontal_major;
	ww += _type_count[2] * _slider_width_min;
	ww += _type_count[3] * _slider_width_min;

	hh += sl_height_min;
	hh += _spacing_vertical;
	hh += cb_height_min;

}


unsigned int
Sliders_Pad_Layout::label_str_length_px_max (
	const QString & str_n ) const
{
	unsigned int res;
	res = std::ceil ( _labels->fontMetrics().width ( str_n ) );
	res = qMin ( res, _labels_vars.max_str_length_px );
	return res;
}



int
Sliders_Pad_Layout::labels_height_hint ( ) const
{
	int str_len_px = 0;
	if ( _widgets_groups.size() > 0 ) {
		for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
			Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
			int len ( label_str_length_px_max ( spwg->group_name() ) );
			str_len_px = qMax ( len, str_len_px );
		}
	} else {
		str_len_px = _labels_vars.max_str_length_px;
	}
	str_len_px += _labels_vars.pad_left;
	str_len_px += _labels_vars.pad_right;

	qreal hh ( _labels_vars.spacing_bottom );
	hh += str_len_px * _labels_vars.angle_sin;
	hh += _labels->fontMetrics().height() * _labels_vars.angle_cos;

	return std::ceil ( hh );
}


void
Sliders_Pad_Layout::calc_label_angle (
	int full_width_n )
{
	const int lbl_v_dist (
		_labels->fontMetrics().height() +
		_labels_vars.spacing_inter );
	const int col_h_dist ( _group_min_width );

	if ( ( col_h_dist >= lbl_v_dist ) &&
		( full_width_n > _input_area_min_width ) )
	{
		_labels_vars.angle = std::asin ( double ( lbl_v_dist ) / double ( col_h_dist ) );
	} else {
		// 90 degree
		_labels_vars.angle = std::asin ( 1.0 );
	}

	_labels_vars.angle_sin = std::sin ( _labels_vars.angle );
	_labels_vars.angle_cos = std::cos ( _labels_vars.angle );

	const double fh2 ( _labels->fontMetrics().height() / 2.0 );
	_labels_vars.center_x = _labels_vars.angle_sin * fh2;
	_labels_vars.center_y = _labels_vars.angle_cos * fh2;

	if ( _labels_vars.center_x < 0.001 ) {
		_labels_vars.center_x = 0;
	}
	if ( _labels_vars.center_y < 0.001 ) {
		_labels_vars.center_y = 0;
	}
}


int
Sliders_Pad_Layout::calc_label_max_x (
	int idx_n )
{
	const Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[idx_n] );
	int str_len_px ( label_str_length_px_max ( spwg->group_name() ) );
	str_len_px += _labels_vars.pad_left;
	str_len_px += _labels_vars.pad_right;

	qreal x_max = spwg->group_pos();
	x_max += qreal ( spwg->group_width() ) / 2.0;
	x_max += _labels_vars.center_x;
	x_max += str_len_px * _labels_vars.angle_cos;

	return std::ceil ( x_max );
}


int
Sliders_Pad_Layout::calc_labels_max_x ( )
{
	int res ( 0 );

	const int num_cg ( _widgets_groups.size() );
	if ( num_cg > 0 ) {
		res = qMax ( res, calc_label_max_x ( num_cg - 1 ) );
	}
	if ( num_cg > 1 ) {
		res = qMax ( res, calc_label_max_x ( num_cg - 2 ) );
	}
	if ( num_cg > 2 ) {
		res = qMax ( res, calc_label_max_x ( num_cg - 3 ) );
	}
	return res;
}


void
Sliders_Pad_Layout::calc_column_sizes (
	const QRect & rect_n )
{
	const int width_max ( rect_n.width() );
	int width_min ( ( width_max * 4 ) / 5 );
	width_min = qMax ( _input_area_min_width, width_min );

	int ww ( width_max );
	while ( ww >= width_min ) {
		calc_column_widths ( ww, rect_n.height() );
		calc_label_angle ( ww );
		const int max_x = calc_labels_max_x();
		if ( max_x < width_max ) {
			break;
		}
		--ww;
	}
}


void
Sliders_Pad_Layout::calc_column_widths (
	const int width_n,
	const int height_n )
{
	//std::cout << "Sliders_Pad_Layout::calc_column_widths " << width_n << "x" << height_n << "\n";

	const unsigned int num_cg ( _widgets_groups.size() );
	const unsigned int sl_max_width ( height_n / 5 );

	unsigned int group_min_width = width_n;

	if ( num_cg == 0 ) {
		return;
	}

	if ( ( _type_count[2] == 0 ) && ( _type_count[3] == 0 ) ) {
		return;
	}

	unsigned long weights_single[4];
	unsigned long col_widths[4];
	unsigned long col_min_widths[4];

	// Init values
	weights_single[0] = 4; // Spacing minor
	weights_single[1] = 7; // Spacing major
	weights_single[2] = weights_single[1] * 2; // Slider minor
	weights_single[3] = weights_single[1] * 3; // Slider major

	set_weights ( col_widths, 0 );

	col_min_widths[0] = _min_spacing_horizontal_minor;
	col_min_widths[1] = _min_spacing_horizontal_major;
	col_min_widths[2] = _slider_width_min;
	col_min_widths[3] = _slider_width_min;

	const unsigned long free_space ( width_n );
	unsigned long used_space ( 0 );
	unsigned long space_delta;

	// Calculate a first estimate for the column widths
	{
		unsigned long weights_all[4];
		for ( int ii=0; ii < 4; ++ii ) {
			weights_all[ii] = weights_single[ii] * _type_count[ii];
		}

		unsigned long weights_total ( sum_up ( weights_all ) );

		// Calculate first estimate for the column widths
		for ( int ii=0; ii < 4; ++ii ) {
			if ( _type_count[ii] > 0 ) {
				col_widths[ii] = ( weights_all[ii] * free_space ) / ( weights_total * _type_count[ii] );
				col_widths[ii] = qMax ( col_widths[ii], col_min_widths[ii] );
			} else {
				col_widths[ii] = 0;
			}
		}

	}

	// Check for maximum slider width
	for ( int ii=2; ii < 4; ++ii ) {
		if ( col_widths[ii] > sl_max_width ) {
			col_widths[ii] = sl_max_width;
		}
	}

	used_space = 0;
	for ( int ii=0; ii < 4; ++ii ) {
		used_space += col_widths[ii] * _type_count[ii];
	}

	// Reduce used space if too much has been taken
	for ( int ii=0; ii < 100; ++ii ) {
		if ( used_space <= free_space ) {
			break;
		}

		// The estimate is too big
		for ( int ii=0; ii < 4; ++ii ) {
			if ( _type_count[ii] > 0 ) {
				if ( col_widths[ii] > col_min_widths[ii] ) {
					col_widths[ii] -= 1;
					used_space -= _type_count[ii];
				} else {
					col_widths[ii] = col_min_widths[ii];
				}
			}
		}
	}

	space_delta = free_space - used_space;


	// Debug messages
	//std::cout << "-- Begin --"  << "\n";
	//std::cout << "Width " << width_n << "\n";
	//for ( unsigned int ii=0; ii < 4; ++ii ) {
	//	std::cout << "Num items " << ii << ": " << _type_count[ii] << "\n";
	//}
	//for ( unsigned int ii=0; ii < 4; ++ii ) {
	//	std::cout << "Col widths " << ii << ": " << col_widths[ii] << "\n";
	//}
	//std::cout << "Free space " << free_space << "\n";
	//std::cout << "space_delta " << space_delta << "\n";


	// Check if there's unused space - Expand as much as possible
	if ( used_space < free_space ) {

		space_delta = free_space - used_space;

		// Ajust and normalize single weights for comparison
		for ( int ii=0; ii < 4; ++ii ) {
			if ( _type_count[ii] == 0 ) {
				weights_single[ii] = 0;
			}
		}
		normalize_weights ( weights_single );

		//unsigned int turns ( 0 );
		while ( space_delta > 0 ) {
			//++turns;
			long weights_delta[4];
			unsigned int weights_sorted[4];
			calc_weights_delta ( weights_delta, weights_single, col_widths );
			calc_weights_delta_sorted ( weights_sorted, weights_delta );

			// Find maximum delta
			int increment_idx ( -1 );

			for ( int ii=0; ii < 4; ++ii ) {
				int idx ( weights_sorted[3-ii] );
				if ( ( _type_count[idx] > 0 ) &&
					( _type_count[idx] <= space_delta ) &&
					( weights_delta[idx] >= 0 ) )
				{
					if ( ( idx < 2 ) ||
						( col_widths[idx] < sl_max_width ) )
					{
						increment_idx = idx;
						break;
					}
				}
			}

			// Add to major spacing as a fallback
			if ( increment_idx < 0 ) {
				if ( ( _type_count[1] > 0 ) &&
					( _type_count[1] <= space_delta ) )
				{
					increment_idx = 1;
				}
			}

			if ( increment_idx >= 0 ) {
				col_widths[increment_idx] += 1;
				used_space += _type_count[increment_idx];
				space_delta = free_space - used_space;
			} else {
				break;
			}
		}

		//std::cout << "Turns " << turns << "\n";
	}

	space_delta = free_space - used_space;


	// Debug messages
	//std::cout << "-- End --" << "\n";
	//std::cout << "width_n " << width_n << "\n";
	//for ( unsigned int ii=0; ii < 4; ++ii ) {
	//	std::cout << "Col widths " << ii << ": " << col_widths[ii] << "\n";
	//}
	//used_space = 0;
	//for ( int ii=0; ii < 4; ++ii ) {
	//	used_space += col_widths[ii] * _type_count[ii];
	//}
	//std::cout << "used_space  " << used_space << "\n";
	//std::cout << "space_delta " << space_delta << "\n";
	//std::cout << "Diff " << width_n - ( used_space + space_delta ) << "\n";


	// Write columns positioning data

	unsigned int pos ( col_widths[1] / 2 ); // Spacing major

	for ( unsigned int ii=0; ii < num_cg; ++ii ) {
		Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );

		const unsigned int num_columns ( spwg->widgets().size() );

		unsigned int group_pos ( pos );
		unsigned long group_width ( 0 );

		unsigned long col_width_cur;
		if ( num_columns == 1 ) {
			col_width_cur = col_widths[3]; // Major slider
		} else {
			col_width_cur = col_widths[2]; // Minor slider
		}

		for ( unsigned int jj=0; jj < num_columns; ++jj ) {
			Sliders_Pad_Widgets * spw ( spwg->widgets()[jj] );

			if ( jj > 0 ) {
				pos += col_widths[0]; // Spacing minor
			}
			spw->set_pos ( pos );
			spw->set_width ( col_width_cur );

			pos += col_width_cur;
		}

		group_width = ( pos - group_pos );

		spwg->set_group_pos ( group_pos );
		spwg->set_group_width ( group_width );

		pos += col_widths[1]; // Spacing major
		if ( ii >= ( num_cg - 1 - space_delta ) ) {
			++pos;
		}

		if ( ( group_width > 0 ) && ( group_width < group_min_width ) ) {
			group_min_width = group_width;
		}
	}

	_group_min_width = group_min_width + col_widths[1];
}




void
Sliders_Pad_Layout::update_widgets_geometry ( )
{
	//std::cout << "Sliders_Pad_Layout::update_widgets_geometry\n";

	const QRect con_rect ( _parent->contentsRect() );

	calc_column_sizes ( con_rect );

	const int h_labels_hint ( labels_height_hint() );
	int h_labels = h_labels_hint;
	int h_inputs = con_rect.height() - h_labels;

	// Labels should always be smaller than the sliders
	if ( ( h_labels * 3 ) > ( h_inputs * 1 ) ) {
		h_labels = qMin ( con_rect.height() / 3, h_labels_hint );
		h_inputs = con_rect.height() - h_labels;
	}

	// Check sliders minimum size
	if ( h_inputs < _input_area_min_height ) {
		h_inputs = _input_area_min_height;
		h_labels = con_rect.height() - h_inputs;
	}

	// Labels area geometry
	{
		QRect wdg_area ( con_rect );
		wdg_area.setHeight ( h_labels );
		_labels->setGeometry ( wdg_area );
	}


	// Sliders area
	const int sla_left ( con_rect.left() );
	const int sla_top ( con_rect.top() + _labels->height() );
	const int sla_height ( con_rect.height() - _labels->height() -
		_spacing_vertical - switch_height() );

	// Check box area
	const int cb_height ( switch_height() );
	const int cba_left ( con_rect.left() );
	const int cba_top ( con_rect.top() + con_rect.height() - cb_height );

	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
		bool put_middle;

		// Sliders
		put_middle = false;
		if ( ( spwg->num_sliders() == 1 ) &&
			( spwg->num_sliders() < spwg->num_switches() ) )
		{
			put_middle = true;
		}

		for ( int jj=0; jj < spwg->widgets().size(); ++jj ) {
			Sliders_Pad_Widgets * spw ( spwg->widgets()[jj] );

			if ( spw->slider() == 0 ) {
				continue;
			}

			int ll ( sla_left  );
			if ( put_middle ) {
				ll += spwg->group_pos();
				ll += ( spwg->group_width() - spw->width() ) / 2;
			} else {
				ll += spw->pos();
			}

			const QRect re_wdg (
				ll, sla_top,
				spw->width(), sla_height );

			spw->slider()->setGeometry ( re_wdg );
		}

		// Switches
		put_middle = false;
		if ( ( spwg->num_switches() == 1 ) &&
			( spwg->num_switches() < spwg->num_sliders() ) )
		{
			put_middle = true;
		}

		for ( int jj=0; jj < spwg->widgets().size(); ++jj ) {
			Sliders_Pad_Widgets * spw ( spwg->widgets()[jj] );

			if ( spw->switch_wdg() == 0 ) {
				continue;
			}

			int cb_w ( cb_height );
			int cb_l ( cba_left );

			if ( put_middle ) {
				// Center switch in column group
				unsigned int xoff ( 0 );

				if ( spwg->group_width() > cb_w ) {
					xoff = spwg->group_width() - cb_w;
				}
				if ( ( xoff % 2 ) > 0 ) {
					++cb_w;
				}
				xoff /= 2;

				cb_l += spwg->group_pos();
				cb_l += xoff;
			} else {
				// Center switch in column
				unsigned int xoff ( 0 );

				if ( spw->width() > cb_w ) {
					xoff = spw->width() - cb_w;
				}
				if ( ( xoff % 2 ) > 0 ) {
					++cb_w;
				}
				xoff /= 2;

				cb_l += spw->pos();
				cb_l += xoff;
			}

			const QRect re_cb (
				cb_l,
				cba_top,
				cb_w,
				cb_height );

			spw->switch_wdg()->setGeometry ( re_cb );
		}

		spwg->set_all_visible ( true );
	}
}





} // End of namespace

