//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "snd_controls_model.hpp"

#include <QFont>
#include <iostream>


namespace QSnd
{


Snd_Controls_Model::Snd_Controls_Model ( ) :
_default_card_idx ( -1 )
{
	QStandardItem * item_root ( invisibleRootItem() );

	QFont fnt;
	fnt.setWeight ( QFont::Bold );

	_item_cards = new QStandardItem;
	_item_cards->setText ( tr ( "Cards" ) );
	_item_cards->setFont ( fnt );
	_item_cards->setEditable ( false );
	_item_cards->setSelectable ( false );
	item_root->appendRow ( _item_cards );

	_item_plugs = new QStandardItem;
	_item_plugs->setText ( tr ( "Plugins" ) );
	_item_plugs->setFont ( fnt );
	_item_plugs->setEditable ( false );
	_item_plugs->setSelectable ( false );
	item_root->appendRow ( _item_plugs );
}


Snd_Controls_Model::~Snd_Controls_Model ( )
{
	clear_cards_and_plugins();
	snd_config_update_free_global();
}


const QSnd::Snd_Card_Info *
Snd_Controls_Model::card_info (
	int index_n ) const
{
	if ( index_n < num_cards() ) {
		return _card_infos[index_n];
	}
	return 0;
}


const QSnd::Snd_Control_Info *
Snd_Controls_Model::ctl_info (
	const QModelIndex & idx_n ) const
{
	const QSnd::Snd_Control_Info * res ( 0 );
	if ( idx_n.parent() == _item_cards->index() ) {
		if ( idx_n.row() < _controls_cards.size() ) {
			res = &_controls_cards[ idx_n.row() ];
		}
	} else if ( idx_n.parent() == _item_plugs->index() ) {
		if ( idx_n.row() < _controls_plugs.size() ) {
			res = &_controls_plugs[ idx_n.row() ];
		}
	}

	return res;
}


QModelIndex
Snd_Controls_Model::ctl_info_index (
	const QSnd::Snd_Control_Info & ctl_info_n ) const
{
	QModelIndex res;

	// Search in cards
	for ( int ii=0; ii < _controls_cards.size(); ++ii ) {
		if ( _controls_cards[ii] == ctl_info_n ) {
			res = _item_cards->child ( ii )->index();
			break;
		};
	}

	// Search in plugins
	if ( !res.isValid() ) {
		for ( int ii=0; ii < _controls_plugs.size(); ++ii ) {
			if ( _controls_plugs[ii] == ctl_info_n ) {
				res = _item_plugs->child ( ii )->index();
				break;
			};
		}
	}

	return res;
}


void
Snd_Controls_Model::update_data ( )
{
	emit layoutAboutToBeChanged();

	clear_cards_and_plugins ( );

	// Clear snd_config completely and reload it
	snd_config_update_free_global();
	int err = snd_config_update();

	if ( ( err < 0 ) || ( snd_config == 0 ) ) {
		std::cerr << "[EE] Alsa configuration reading failed\n";
		print_alsa_error ( "snd_config_update()", err );
	} else {
		load_cards();
		load_plugins();
	}

	emit layoutChanged();
}



void
Snd_Controls_Model::clear_cards_and_plugins ( )
{
	_item_cards->removeRows ( 0, _item_cards->rowCount() );
	_item_plugs->removeRows ( 0, _item_plugs->rowCount() );

	if ( _card_infos.size() > 0 ) {
		for ( int ii=0; ii < _card_infos.size(); ++ii ) {
			delete _card_infos[ii];
		}
		_card_infos.clear();
	}
	_controls_cards.clear();
	_controls_plugs.clear();

	_default_index = QModelIndex();
	_default_ctl.clear();
	_default_card_idx = -1;
}



void
Snd_Controls_Model::load_cards ( )
{
	_default_card_idx = qMax ( 0, acquire_alsa_default_card() );

	int card_idx = -1;
	while ( ( snd_card_next ( &card_idx ) == 0 ) && ( card_idx >= 0 ) ) {
		QSnd::Snd_Card_Info * card ( new QSnd::Snd_Card_Info );
		card->read_info ( card_idx );
		_card_infos.push_back ( card );
	}

	_controls_cards.resize ( _card_infos.size() );

	for ( int ii=0; ii < _card_infos.size(); ++ii ) {
		Snd_Control_Info & ctl ( _controls_cards[ii] );
		const Snd_Card_Info * cinfo ( _card_infos[ii] );

		Snd_Control_Argument argm ( "CARD", "string" );
		argm.arg_value = QString ( "%1" ).arg ( cinfo->card_index() );

		ctl.set_ctl_name ( "hw" );
		ctl.append_arg ( argm );

		if ( cinfo->card_index() == _default_card_idx ) {
			_default_ctl = ctl;
		}
	}


	// Create standard items
	if ( _controls_cards.size() > 0 ) {
		for ( int ii=0; ii < _controls_cards.size(); ++ii ) {
			const Snd_Card_Info * cinfo ( _card_infos[ii] );

			QStandardItem * sitem ( new QStandardItem );
			sitem->setText ( cinfo->card_name() );
			sitem->setEditable ( false );

			if ( cinfo->card_index() == _default_card_idx ) {
				QFont fnt;
				fnt.setStyle ( QFont::StyleItalic );
				sitem->setFont ( fnt );
			}

			{
				QString ttip;
				ttip += "<div><b>";
				ttip += tr ( "Name" );
				ttip += ":</b> ";
				ttip += cinfo->card_name();
				ttip += "</div>\n";

				ttip += "<div><b>";
				ttip += tr ( "Id" );
				ttip += ":</b> ";
				ttip += cinfo->card_id();
				ttip += "</div>\n";

				ttip += "<div><b>";
				ttip += tr ( "Mixer name" );
				ttip += ":</b> ";
				ttip += cinfo->card_mixer_name();
				ttip += "</div>\n";

				if ( cinfo->card_index() == _default_card_idx ) {
					ttip += "<div><i>";
					ttip += tr ( "Default card" );
					ttip += "</i></div>";
				}
				sitem->setToolTip ( ttip );
			}

			_item_cards->appendRow ( sitem );

			if ( cinfo->card_index() == _default_card_idx ) {
				_default_index = sitem->index();
			}
		}
	}
}


void
Snd_Controls_Model::load_plugins ( )
{
	int err ( 0 );

	snd_config_t * snd_cfg_ctl ( 0 );

	err = snd_config_search	(
		snd_config, "ctl", &snd_cfg_ctl );

	if ( ( err >= 0 ) && ( snd_cfg_ctl != 0 ) ) {

		snd_config_iterator_t iter;

		iter = snd_config_iterator_first ( snd_cfg_ctl );
		while ( iter != snd_config_iterator_end ( snd_cfg_ctl ) ) {
			snd_config_t * snd_cfg_entry (
				snd_config_iterator_entry ( iter) );
			if ( snd_config_get_type ( snd_cfg_entry ) ==
				SND_CONFIG_TYPE_COMPOUND )
			{
				const char * pstr ( 0 );
				snd_config_get_id ( snd_cfg_entry, &pstr );
				if ( pstr != 0 ) {
					Snd_Control_Info qctli ( pstr );
					_controls_plugs.append ( qctli );
				}
				//std::cout << "pstr " << pstr << "\n";
			}

			iter = snd_config_iterator_next ( iter );
		}
	}


	// Create standard items
	if ( _controls_plugs.size() > 0 ) {
		for ( int ii=0; ii < _controls_plugs.size(); ++ii ) {
			QStandardItem * sitem ( new QStandardItem );
			sitem->setText ( _controls_plugs[ii].ctl_name() );
			sitem->setEditable ( false );

			_item_plugs->appendRow ( sitem );
		}
	}

}


} // End of namespace

