//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "snd_mixer_simple.hpp"

#include <iostream>
#include <QEvent>
#include <QCoreApplication>


namespace QSnd
{


//
// Snd_Mixer_Simple
//
Snd_Mixer_Simple::Snd_Mixer_Simple (
	QObject * parent_n ) :
QObject ( parent_n ),
_snd_hctl ( 0 ),
_snd_mixer ( 0 ),
_update_requested ( false ),
_signal_changes_requested ( false )
{
}


Snd_Mixer_Simple::~Snd_Mixer_Simple ( )
{
	close_mixer();
}



void
Snd_Mixer_Simple::close_mixer ( )
{

	// Destroy socket notifiers
	for ( int ii=0; ii < _socket_notifiers.size(); ++ii ) {
		delete _socket_notifiers[ii];
	}
	_socket_notifiers.clear();
	_pollfds.clear();


	if ( _mixer_simple_elems.size() > 0 ) {
		// Disconnect mixer elements
		for ( int ii=0; ii < _mixer_simple_elems.size(); ++ii ) {
			snd_mixer_elem_t * elem ( _mixer_simple_elems[ii]->snd_mixer_elem() );
			snd_mixer_elem_set_callback ( elem, 0 );
			snd_mixer_elem_set_callback_private ( elem, 0 );
		}

		// Destroy mixer elements
		for ( int ii=0; ii < _mixer_simple_elems.size(); ++ii ) {
			delete _mixer_simple_elems[ii];
		}
		_mixer_simple_elems.clear();
	}


	if ( snd_mixer() != 0 ) {
		snd_mixer_set_callback ( snd_mixer(), 0 );
		snd_mixer_set_callback_private ( snd_mixer(), 0 );

		if ( _snd_hctl != 0 ) {
			snd_mixer_detach_hctl ( snd_mixer(), _snd_hctl );
		}

		snd_mixer_close ( snd_mixer() );
		_snd_mixer = 0;
	}

	if ( _snd_hctl != 0 ) {
		snd_hctl_close ( _snd_hctl );
		_snd_hctl = 0;
	}

}


int
Snd_Mixer_Simple::open_mixer (
	const QString & dev_name_n )
{
	int err;

	close_mixer();

	err = snd_mixer_open ( &_snd_mixer, 0 );
	if ( err < 0 ) {
		QString err_msg ( "snd_mixer_open" );
		print_snd_error ( err_msg, err );
		return err;
	}

	err = snd_mixer_selem_register ( _snd_mixer, NULL, NULL );
	if ( err < 0 ) {
		QString err_msg ( "snd_mixer_selem_register" );
		print_snd_error ( err_msg, err );
		close_mixer();
		return err;
	}

	// Error string
	QString dev_err_str ( " ( %1 )" );
	dev_err_str = dev_err_str.arg ( dev_name_n );

	// Open control handle
	err = snd_hctl_open (
		&_snd_hctl,
		dev_name_n.toLocal8Bit().constData(),
		SND_CTL_NONBLOCK );

	if ( err < 0 ) {
		QString err_msg ( "snd_hctl_open" + dev_err_str );
		print_snd_error ( err_msg, err );
		close_mixer();
		return err;
	}

	// Attach hctl handle to mixer
	err = snd_mixer_attach_hctl ( _snd_mixer, _snd_hctl );
	if ( err < 0 ) {
		_snd_hctl = 0;
		QString err_msg ( "snd_mixer_attach_hctl" + dev_err_str );
		print_snd_error ( err_msg, err );
		close_mixer();
		return err;
	}

	// Load mixer
	err = snd_mixer_load ( _snd_mixer );
	if ( err < 0 ) {
		QString err_msg ( "snd_mixer_load" + dev_err_str );
		print_snd_error ( err_msg, err );
		close_mixer();
		return err;
	}

	snd_mixer_set_callback ( snd_mixer(), alsa_callback_mixer );
	snd_mixer_set_callback_private ( snd_mixer(), this );

	// Create mixer elements
	err = create_qsnd_mixer_selems();
	if ( err < 0 ) {
		close_mixer();
		return err;
	}

	// Create socket notifiers
	err = create_socket_notifiers();
	if ( err < 0 ) {
		close_mixer();
		return err;
	}

	return 0;
}


int
Snd_Mixer_Simple::create_qsnd_mixer_selems ( )
{
	// Create and connect Snd_Mixer_Simple_Elem items
	snd_mixer_elem_t * elem;
	snd_mixer_selem_id_t * elem_id;
	snd_mixer_selem_id_alloca ( &elem_id );

	elem = snd_mixer_first_elem ( _snd_mixer );
	while ( elem != 0 )	{
		snd_mixer_selem_get_id ( elem, elem_id );
		if ( filter_mixer_elem ( elem ) == 0 ) {
			Snd_Mixer_Simple_Elem * qsme =
				new Snd_Mixer_Simple_Elem ( this, _snd_mixer, elem_id );

			snd_mixer_elem_set_callback_private ( elem, qsme );
			snd_mixer_elem_set_callback ( elem, alsa_callback_mixer_elem );

			connect ( qsme, SIGNAL ( sig_element_changed() ),
				this, SLOT ( element_changed() ) );

			_mixer_simple_elems.append ( qsme );
		}
		elem = snd_mixer_elem_next ( elem );
	}


	// Handle names that occur multiple times
	const QString str_new ( "%1, %2" );
	for ( int ii=0; ii < _mixer_simple_elems.size(); ++ii ) {
		bool dbl ( false );

		Snd_Mixer_Simple_Elem * qsme ( _mixer_simple_elems[ii] );
		QString str_cur ( qsme->display_name() );

		// Detect if there're doubles at all
		for ( int jj=ii+1; jj < _mixer_simple_elems.size(); ++jj ) {
			Snd_Mixer_Simple_Elem * qsme2 ( _mixer_simple_elems[jj] );
			if ( qsme2->display_name() == str_cur )
			{
				dbl = true;
				break;
			}
		}

		// Rename doubles
		if ( dbl ) {
			for ( int jj=ii; jj < _mixer_simple_elems.size(); ++jj ) {
				Snd_Mixer_Simple_Elem * qsme2 ( _mixer_simple_elems[jj] );
				if ( qsme2->display_name() == str_cur ) {
					QString nname ( str_new );
					nname = nname.arg ( str_cur );
					nname = nname.arg ( qsme2->elem_index() );
					qsme2->set_display_name ( nname );
				}
			}
		}
	}

	update_all_mixer_values();

	return 0;
}


int
Snd_Mixer_Simple::filter_mixer_elem (
	snd_mixer_elem_t * elem_n )
{
	int res ( 0 );
	QString el_name ( snd_mixer_selem_get_name ( elem_n ) );
	QString el_name_low ( el_name.toLower() );

	if ( el_name_low == "eq peaks" ) {
		res = 1;
	}
	return res;
}


int
Snd_Mixer_Simple::create_socket_notifiers ( )
{
	//std::cout << "Snd_Mixer_Simple::create_socket_notifiers" << "\n";

	// Setup socket notifiers
	int num_fds;
	int err;
	num_fds = snd_mixer_poll_descriptors_count ( _snd_mixer );
	_pollfds.resize ( num_fds );
	memset ( _pollfds.data(), 0, num_fds*sizeof ( pollfd ) );

	err = snd_mixer_poll_descriptors ( _snd_mixer, _pollfds.data(), num_fds );
	if ( err < 0 ) {
		print_alsa_error ( "snd_mixer_poll_descriptors", err );
		return err;
	}

	for ( int ii=0; ii < num_fds; ++ii ) {
		int fd ( _pollfds[ii].fd );
		if ( fd != 0 ) {
			QSocketNotifier * sn (
				new QSocketNotifier ( fd, QSocketNotifier::Read, this ) );

			connect ( sn, SIGNAL ( activated ( int ) ),
				this, SLOT ( socket_event ( int ) ) );

			_socket_notifiers.append ( sn );
		}
	}

	return 0;
}


void
Snd_Mixer_Simple::set_socked_notifiers_enabled (
	bool flag_n )
{
	for ( int ii=0; ii < _socket_notifiers.size(); ++ii ) {
		_socket_notifiers[ii]->setEnabled ( flag_n );
	}
}


bool
Snd_Mixer_Simple::update_all_mixer_values ( )
{
	bool changed ( false );
	for ( int ii=0; ii < _mixer_simple_elems.size(); ++ii ) {
		if ( _mixer_simple_elems[ii]->update_mixer_values() ) {
			changed = true;
		};
	}
	return changed;
}


bool
Snd_Mixer_Simple::event (
	QEvent * event_n )
{
	bool res ( true );

	if ( event_n->type() == event_type_update_request ) {
		//std::cout << "Snd_Mixer_Simple::event: " << "Update requested" << "\n";
		if ( !_update_requested ) {
			_update_requested = true;
			QCoreApplication::postEvent (
				this, new QEvent ( event_type_update ) );
		}

	} else if ( event_n->type() == event_type_update ) {
		//std::cout << "Snd_Mixer_Simple::event: " << "Updating mixer values" << "\n";
		_update_requested = false;
		if ( update_all_mixer_values() ) {
			emit sig_mixer_values_changed();
		}

	} else if ( event_n->type() == event_type_element_changed ) {
		//std::cout << "Snd_Mixer_Simple::event: " << "Element changed" << "\n";
		if ( !_signal_changes_requested ) {
			_signal_changes_requested = true;
			QCoreApplication::postEvent (
				this, new QEvent ( event_type_signal_elements_change ) );
		}

	} else if ( event_n->type() == event_type_signal_elements_change ) {
		//std::cout << "Snd_Mixer_Simple::event: " << "Signaling a mixer elements change" << "\n";
		_signal_changes_requested = false;
		emit sig_mixer_elements_changed();

	} else {
		res = QObject::event ( event_n );
	}

	return res;
}


void
Snd_Mixer_Simple::socket_event (
	int )
{
	if ( _snd_mixer != 0 ) {
		int num_ev ( snd_mixer_handle_events ( _snd_mixer ) );
		if ( num_ev != 0 ) {
			if ( num_ev > 0 ) {
				emit sig_mixer_values_changed();
			} else {
				// Error. Disable socket notifiers
				set_socked_notifiers_enabled ( false );
				print_alsa_error ( "snd_mixer_handle_events ", num_ev );
			}
		}
	}
}


void
Snd_Mixer_Simple::element_changed ( )
{
	QEvent ev_change ( event_type_element_changed );
	QCoreApplication::sendEvent ( this, &ev_change );
}


//
// Alsa callbacks
//

int
alsa_callback_mixer (
	snd_mixer_t * snd_mixer_n,
	unsigned int mask_n,
	snd_mixer_elem_t * )
{
	int res ( 0 );

	//std::cout << "alsa_callback_mixer " << mask_n << "\n";

	Snd_Mixer_Simple * qsnd_mixer;
	{
		void * priv ( snd_mixer_get_callback_private ( snd_mixer_n ) );
		qsnd_mixer = reinterpret_cast < Snd_Mixer_Simple * > ( priv );
	}

	if ( qsnd_mixer != 0 ) {

		switch ( mask_n ) {
			case SND_CTL_EVENT_MASK_ADD :
			case SND_CTL_EVENT_MASK_INFO :
			case SND_CTL_EVENT_MASK_REMOVE:
			case SND_CTL_EVENT_MASK_TLV :
			case SND_CTL_EVENT_MASK_VALUE:
				qsnd_mixer->element_changed();
				break;

			default:
				std::cerr << "alsa_callback_mixer: Unknown mask ( " << mask_n << " )\n";
				res = -1;
				break;
		}
	}

	return res;
}


} // End of namespace

