//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_snd_mixer_simple_elem_hpp__
#define __INC_snd_mixer_simple_elem_hpp__

#include "alsa.hpp"

#include <vector>
#include <QObject>
#include <QString>


namespace QSnd
{


///
/// @brief Snd_Mixer_Simple_Elem
///
/// Brings Qt and ALSA objects together but without
/// any GUI objects
///
class Snd_Mixer_Simple_Elem :
    public QObject
{
	Q_OBJECT

	// Public typedefs
	public:

	typedef std::vector < snd_mixer_selem_channel_id_t > Channel_Buffer;
	typedef std::vector < long > Volume_Buffer;
	typedef std::vector < bool > Switch_Buffer;

	typedef unsigned int Enum_Index;
	typedef std::vector < Enum_Index > Enum_Buffer;
	typedef std::vector < QString > Enum_Names_Buffer;

	typedef std::vector < long > DB_Buffer;


	// Public methods
	public:

	Snd_Mixer_Simple_Elem (
		QObject * parent_n = 0,
		snd_mixer_t * mixer_n = 0,
		snd_mixer_selem_id_t * elem_id_n = 0 );

	~Snd_Mixer_Simple_Elem ( );

	void
	clear ( );


	// Alsa structs access

	snd_mixer_t *
	snd_mixer ( ) const;

	void
	set_snd_mixer (
		snd_mixer_t * mixer_n );


	void
	set_snd_mixer_selem_id (
		snd_mixer_selem_id_t * selem_id_n );

	snd_mixer_selem_id_t *
	snd_mixer_selem_id ( ) const;


	snd_mixer_elem_t *
	snd_mixer_elem ( ) const;




	const QString &
	elem_name ( ) const;

	const QString &
	display_name ( ) const;

	void
	set_display_name (
		const QString & name_n );


	unsigned int
	elem_index ( ) const;

	int
	num_channels (
		int snd_dir_n ) const;


	const char *
	channel_name (
		unsigned int snd_dir_n,
		unsigned int channel_idx_n ) const;


	// Availability

	bool
	has_volume (
		int snd_dir_n ) const;

	bool
	has_dB (
		int snd_dir_n ) const;

	bool
	has_switch (
		int snd_dir_n ) const;

	bool
	has_enum (
		int snd_dir_n ) const;


	// Joined feature

	bool
	volume_joined (
		int snd_dir_n ) const;

	bool
	switch_joined (
		int snd_dir_n ) const;

	bool
	enum_joined (
		int snd_dir_n ) const;


	bool
	is_active ( ) const;


	// Equality

	bool
	volumes_equal (
		int snd_dir_n ) const;

	bool
	switches_equal (
		int snd_dir_n ) const;

	bool
	enums_equal (
		int snd_dir_n ) const;


	long
	volume_min (
		int snd_dir_n ) const;

	long
	volume_max (
		int snd_dir_n ) const;


	long
	dB_min (
		int snd_dir_n ) const;

	long
	dB_max (
		int snd_dir_n ) const;

	long
	ask_dB_vol (
		int snd_dir_n,
		long dB_value_n,
		int dir_n = -1 );

	long
	ask_vol_dB (
		int snd_dir_n,
		long volume_n );

	long
	ask_dB_vol_nearest (
		int snd_dir_n,
		long dB_value_n );


	unsigned int
	enum_num_items ( ) const;

	const Enum_Names_Buffer &
	enum_item_names ( ) const;


	// Buffers

	const Channel_Buffer &
	channel_buffer (
		int snd_dir_n ) const;

	const Volume_Buffer &
	volume_buffer (
		int snd_dir_n ) const;

	const DB_Buffer &
	dB_buffer (
		int snd_dir_n ) const;

	const Switch_Buffer &
	switch_buffer (
		int snd_dir_n ) const;

	const Enum_Buffer &
	enum_buffer (
		int snd_dir_n ) const;


	// Volume setting

	void
	set_volume (
		int snd_dir_n,
		int channel_idx_n,
		long volume_n );

	void
	set_volume_all (
		int snd_dir_n,
		long volume_n );

	void
	level_volumes (
		int snd_dir_n );


	// Decibel setting

	void
	set_dB (
		int snd_dir_n,
		int channel_idx_n,
		long dB_val_n,
		int dir_n = -1 );

	void
	set_dB_all (
		int snd_dir_n,
		long dB_val_n,
		int dir_n = -1 );


	// Switch setting

	void
	set_switch (
		int snd_dir_n,
		int channel_idx_n,
		bool index_n );

	void
	set_switch_all (
		int snd_dir_n,
		bool index_n );

	void
	level_switches (
		int snd_dir_n );


	// Enum setting

	void
	set_enum_index (
		int snd_dir_n,
		int channel_idx_n,
		Enum_Index state_n );

	void
	set_enum_index_all (
		int snd_dir_n,
		Enum_Index state_n );


	// Callback methods

	/// @brief Reads all values from alsa
	///
	/// @return true if a value changed
	bool
	update_mixer_values ( );

	/// @brief Signalizes the parent that this element changed
	void
	element_changed ( );


	// Signals
	signals:

	/// @brief Gets emitted when a value was set
	void
	sig_value_set ( );

	/// @brief Gets emitted when a this element changed it's structure
	void
	sig_element_changed ( );


	// Protected methods
	protected:

	void
	value_was_set ( );


	// Private attributes
	private:

	snd_mixer_t * _snd_mixer;

	snd_mixer_selem_id_t * _snd_mixer_selem_id;

	QString _elem_name;
	QString _display_name;

	bool _has_volume[2];
	bool _has_dB[2];
	bool _has_switch[2];
	bool _has_enum[2];

	bool _volume_joined[2];
	bool _switch_joined[2];
	bool _enum_joined[2];

	bool _is_active;

	bool _volumes_equal[2];
	bool _switches_equal[2];
	bool _enums_equal[2];

	long _volume_min[2];
	long _volume_max[2];

	long _dB_min[2];
	long _dB_max[2];

	unsigned int _enum_num_items;
	Enum_Names_Buffer _enum_item_names;

	Channel_Buffer _channels[2];
	Volume_Buffer _volumes[2];
	DB_Buffer _dB_values[2];
	Switch_Buffer _switch_states[2];
	Enum_Buffer _enum_values[2];
};



inline
snd_mixer_t *
Snd_Mixer_Simple_Elem::snd_mixer ( ) const
{
	return _snd_mixer;
}


inline
snd_mixer_selem_id_t *
Snd_Mixer_Simple_Elem::snd_mixer_selem_id ( ) const
{
	return _snd_mixer_selem_id;
}


inline
snd_mixer_elem_t *
Snd_Mixer_Simple_Elem::snd_mixer_elem ( ) const
{
	return snd_mixer_find_selem ( snd_mixer(), snd_mixer_selem_id() );
}


inline
void
Snd_Mixer_Simple_Elem::set_snd_mixer (
	snd_mixer_t * mixer_n )
{
	_snd_mixer = mixer_n;
}


inline
const QString &
Snd_Mixer_Simple_Elem::elem_name ( ) const
{
	return _elem_name;
}


inline
const QString &
Snd_Mixer_Simple_Elem::display_name ( ) const
{
	return _display_name;
}


inline
int
Snd_Mixer_Simple_Elem::num_channels (
	int snd_dir_n ) const
{
	return _channels[snd_dir_n].size();
}



// Availability

inline
bool
Snd_Mixer_Simple_Elem::has_volume (
	int snd_dir_n ) const
{
	return _has_volume[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::has_dB (
	int snd_dir_n ) const
{
	return _has_dB[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::has_switch (
	int snd_dir_n ) const
{
	return _has_switch[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::has_enum (
	int snd_dir_n ) const
{
	return _has_enum[snd_dir_n];
}


// Joined feature

inline
bool
Snd_Mixer_Simple_Elem::volume_joined (
	int snd_dir_n ) const
{
	return _volume_joined[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::switch_joined (
	int snd_dir_n ) const
{
	return _switch_joined[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::enum_joined (
	int snd_dir_n ) const
{
	return _enum_joined[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::is_active ( ) const
{
	return _is_active;
}


// Equality

inline
bool
Snd_Mixer_Simple_Elem::volumes_equal (
	int snd_dir_n ) const
{
	return _volumes_equal[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::switches_equal (
	int snd_dir_n ) const
{
	return _switches_equal[snd_dir_n];
}


inline
bool
Snd_Mixer_Simple_Elem::enums_equal (
	int snd_dir_n ) const
{
	return _enums_equal[snd_dir_n];
}


inline
long
Snd_Mixer_Simple_Elem::volume_min (
	int snd_dir_n ) const
{
	return _volume_min[snd_dir_n];
}


inline
long
Snd_Mixer_Simple_Elem::volume_max (
	int snd_dir_n ) const
{
	return _volume_max[snd_dir_n];
}


inline
long
Snd_Mixer_Simple_Elem::dB_min (
	int snd_dir_n ) const
{
	return _dB_min[snd_dir_n];
}


inline
long
Snd_Mixer_Simple_Elem::dB_max (
	int snd_dir_n ) const
{
	return _dB_max[snd_dir_n];
}


inline
long
Snd_Mixer_Simple_Elem::ask_dB_vol (
	int snd_dir_n,
	long dB_value_n,
	int dir_n )
{
	long res ( 0 );
	if ( snd_dir_n == 0 ) {
		snd_mixer_selem_ask_playback_dB_vol (
			snd_mixer_elem(), dB_value_n, dir_n, &res );
	} else {
		snd_mixer_selem_ask_capture_dB_vol (
			snd_mixer_elem(), dB_value_n, dir_n, &res );
	}
	return res;
}


inline
long
Snd_Mixer_Simple_Elem::ask_vol_dB (
	int snd_dir_n,
	long volume_n )
{
	long res ( 0 );
	if ( snd_dir_n == 0 ) {
		snd_mixer_selem_ask_playback_vol_dB (
			snd_mixer_elem(), volume_n, &res );
	} else {
		snd_mixer_selem_ask_capture_vol_dB (
			snd_mixer_elem(), volume_n, &res );
	}
	return res;
}


inline
unsigned int
Snd_Mixer_Simple_Elem::enum_num_items ( ) const
{
	return _enum_num_items;
}


inline
const Snd_Mixer_Simple_Elem::Enum_Names_Buffer &
Snd_Mixer_Simple_Elem::enum_item_names ( ) const
{
	return _enum_item_names;
}


// Buffers

inline
const Snd_Mixer_Simple_Elem::Channel_Buffer &
Snd_Mixer_Simple_Elem::channel_buffer (
	int snd_dir_n ) const
{
	return _channels[snd_dir_n];
}


inline
const Snd_Mixer_Simple_Elem::Volume_Buffer &
Snd_Mixer_Simple_Elem::volume_buffer (
	int snd_dir_n ) const
{
	return _volumes[snd_dir_n];
}


inline
const Snd_Mixer_Simple_Elem::DB_Buffer &
Snd_Mixer_Simple_Elem::dB_buffer (
	int snd_dir_n ) const
{
	return _dB_values[snd_dir_n];
}


inline
const Snd_Mixer_Simple_Elem::Switch_Buffer &
Snd_Mixer_Simple_Elem::switch_buffer (
	int snd_dir_n ) const
{
	return _switch_states[snd_dir_n];
}


inline
const Snd_Mixer_Simple_Elem::Enum_Buffer &
Snd_Mixer_Simple_Elem::enum_buffer (
	int snd_dir_n ) const
{
	return _enum_values[snd_dir_n];
}


///
/// @brief Alsa callback for mixer element events
///
int
alsa_callback_mixer_elem (
	snd_mixer_elem_t * elem,
	unsigned int mask );



} // End of namespace


#endif


