//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "switches_pad.hpp"

#include <QHBoxLayout>
#include <QVBoxLayout>

#include <QEvent>
#include <QFocusEvent>

#include <iostream>


namespace Wdg
{


//
// Switches_Pad
//
Switches_Pad::Switches_Pad (
	QWidget * parent_n ) :
QFrame ( parent_n ),
_vline ( this ),
_proxies_groups ( 0 )
{
	_vline.setFrameStyle ( QFrame::VLine | QFrame::Plain );
	_vline.hide();

	setFrameStyle ( QFrame::StyledPanel | QFrame::Plain );

	{
		QSizePolicy policy ( sizePolicy() );
		policy.setVerticalPolicy ( QSizePolicy::Fixed );
		policy.setHorizontalPolicy ( QSizePolicy::Preferred );
		setSizePolicy ( policy );
	}

	update_layout();
}


Switches_Pad::~Switches_Pad ( )
{
	set_proxies_groups ( 0 );
}


void
Switches_Pad::set_proxies_groups (
	Switches_Pad_Proxy_Group_List * groups_n )
{
	if ( groups_n != _proxies_groups ) {
		clear_widgets_groups();

		_proxies_groups = groups_n;

		create_widgets_groups();
		update_layout();
	}
}


void
Switches_Pad::clear_widgets_groups ( )
{
	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		delete _widgets_groups[ii];
	}
	_widgets_groups.clear();
}


void
Switches_Pad::create_widgets_groups ( )
{
	//std::cout << "Mixer_Switches::create_widgets_groups\n";

	if ( _proxies_groups == 0 ) {
		return;
	}

	const int num_ev_filters ( 4 );
	QObject * ev_filters[num_ev_filters];

	for ( int ii=0; ii < (*_proxies_groups).size(); ++ii ) {
		Switches_Pad_Proxies_Group * sppg ( (*_proxies_groups)[ii] );

		Switches_Pad_Widgets_Group * spwg (
			new Switches_Pad_Widgets_Group );

		if ( sppg->is_multichannel() ) {
			QLabel * label ( new QLabel ( spwg ) );
			label->setText ( sppg->group_name() );
			label->setToolTip ( sppg->tool_tip() );
			label->setFocusPolicy ( Qt::ClickFocus );
			label->setContextMenuPolicy ( Qt::NoContextMenu );

			label->installEventFilter ( spwg );

			spwg->set_label ( label );
		}

		for ( int jj=0; jj < sppg->proxies().size(); ++jj ) {
			Switches_Pad_Proxy * spp ( sppg->proxy ( jj ) );

			ev_filters[0] = this; // Last filter to pass
			ev_filters[1] = spwg;
			ev_filters[2] = sppg;
			ev_filters[3] = spp; // First filter to pass

			Switches_Pad_Widgets * spw ( new Switches_Pad_Widgets );

			if ( spp->is_enum() ) {

				QLabel * label ( new QLabel ( spwg ) );
				QComboBox * cbox ( new QComboBox ( spwg ) );

				label->setText ( spp->item_name() );
				label->setToolTip ( spp->tool_tip() );

				// Setup selectable items
				for ( int ee=0; ee < spp->enum_num_items(); ++ee ) {
					cbox->addItem ( spp->enum_item_name ( ee ) );
				}
				cbox->setCurrentIndex ( spp->enum_index() );
				cbox->setToolTip ( spp->tool_tip() );

				connect ( cbox, SIGNAL ( currentIndexChanged ( int ) ),
					spp, SLOT ( set_enum_index ( int ) ) );
				connect ( spp, SIGNAL ( sig_enum_index_changed ( int ) ),
					cbox, SLOT ( setCurrentIndex ( int ) ) );

				// Install event filters
				for ( int ii=0; ii < num_ev_filters; ++ii ) {
					cbox->installEventFilter ( ev_filters[ii] );
				}

				spw->set_label_wdg ( label );
				spw->set_input_wdg ( cbox );

			} else if ( spp->is_switch() ) {

				QCheckBox * switch_wdg ( new QCheckBox ( spwg ) );
				if ( sppg->is_multichannel() ) {
					switch_wdg->setText ( spp->item_name() );
				} else {
					switch_wdg->setText ( sppg->group_name() );
				}
				switch_wdg->setChecked ( spp->switch_state() );
				switch_wdg->setToolTip ( spp->tool_tip() );

				connect ( switch_wdg, SIGNAL ( toggled ( bool ) ),
					spp, SLOT ( set_switch_state ( bool ) ) );

				connect ( spp, SIGNAL ( sig_switch_state_changed ( bool ) ),
					switch_wdg, SLOT ( setChecked ( bool ) ) );

				// Install event filters
				for ( int ii=0; ii < num_ev_filters; ++ii ) {
					switch_wdg->installEventFilter ( ev_filters[ii] );
				}

				spw->set_input_wdg ( switch_wdg );
			}

			if ( spw->input_wdg() != 0 ) {
				spwg->append_widgets ( spw );
			} else {
				delete spw;
			}

		}

		// Append widgets group to the list
		if ( spwg->widgets().size() > 0 ) {
			_widgets_groups.append ( spwg );
		} else {
			delete spwg;
		}

	}
}



void
Switches_Pad::update_layout ( )
{
	if ( layout() != 0 ) {
		delete layout();
	}

	QLayout * lay ( 0 );

	const Qt::Alignment align_tl ( Qt::AlignLeft | Qt::AlignTop );
	const int break_count ( 3 );
	bool is_multicol ( _widgets_groups.size() > break_count );

	if ( is_multicol ) {

		QHBoxLayout * lay_hbox = new QHBoxLayout();
		QVBoxLayout * lay_col1 = new QVBoxLayout();
		QVBoxLayout * lay_col2 = new QVBoxLayout();

		lay_hbox->setAlignment ( align_tl );
		lay_col1->setAlignment ( align_tl );
		lay_col1->setAlignment ( align_tl );

		lay_col1->setContentsMargins ( 0, 0, 0, 0 );
		lay_col2->setContentsMargins ( 0, 0, 0, 0 );

		int lim = _widgets_groups.size() / 2;
		lim += _widgets_groups.size() % 2;
		lim = qMax ( break_count, lim );

		for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
			QWidget * wdg ( _widgets_groups[ii] );
			if ( ii < lim ) {
				lay_col1->addWidget ( wdg );
			} else {
				lay_col2->addWidget ( wdg );
			}
		}

		lay_col1->addStretch ( 1 );
		lay_col2->addStretch ( 1 );

		lay_hbox->addLayout ( lay_col1 );
		lay_hbox->addWidget ( &_vline );
		lay_hbox->addLayout ( lay_col2 );

		lay = lay_hbox;
	} else {

		QVBoxLayout * lay_items = new QVBoxLayout();
		lay_items->setAlignment ( align_tl );

		for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
			lay_items->addWidget ( _widgets_groups[ii] );
		}

		lay = lay_items;
	}

	setLayout ( lay );

	_vline.setVisible ( is_multicol );

	updateGeometry();
}


void
Switches_Pad::update_focus_info ( )
{
	_focus_info.has_focus = false;

	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		Switches_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
		Switches_Pad_Widgets * spw ( spwg->focus_widgets() );

		if ( spw != 0 ) {
			_focus_info.has_focus = true;
			_focus_info.group_idx   = ii;
			_focus_info.widgets_idx = spwg->focus_widgets_idx();
			break;
		}
	}
}


void
Switches_Pad::set_focus_proxy (
	unsigned int proxies_group_idx_n,
	unsigned char proxy_idx_n )
{
	const unsigned int num_groups ( _widgets_groups.size() );
	if ( proxies_group_idx_n < num_groups ) {
		Switches_Pad_Widgets_Group * spwg (
			_widgets_groups[ proxies_group_idx_n ] );

		QWidget * wdg ( 0 );

		// Select the slider of given widget set
		if ( int ( proxy_idx_n ) < spwg->widgets().size() ) {
			wdg = spwg->widgets ( proxy_idx_n )->input_wdg();
		}

		// No widget found so far - Take first slider then
		if ( wdg == 0 ) {
			for ( int ii=0; ii < spwg->widgets().size(); ++ii ) {
				Switches_Pad_Widgets * spw ( spwg->widgets ( ii ) );
				if ( spw->input_wdg() != 0 ) {
					wdg = spw->input_wdg();
					break;
				}
			}
		}

		if ( wdg != 0 ) {
			wdg->setFocus();
		}
	}
}


bool
Switches_Pad::eventFilter (
	QObject * ,
	QEvent * event )
{
	QFocusEvent * fev ( dynamic_cast < QFocusEvent * > ( event ) );
	if ( fev != 0 ) {
		if ( fev->lostFocus() ) {
			_focus_info.has_focus = false;
		} else if ( fev->gotFocus() ) {
			update_focus_info();
		}
	}

	return false;
}



} // End of namespace

