//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "tree_view_controls.hpp"

#include <iostream>

namespace QSnd
{


//
// Tree_View_Controls
//
Tree_View_Controls::Tree_View_Controls (
	QWidget * parent ) :
QTreeView ( parent ),
_show_rows_min ( 3 ),
_show_rows_avrg ( 6 ),
_min_str_len ( 16 )
{
	setHeaderHidden ( true );
	setRootIsDecorated ( false );

	update_maximum_size();
}


Tree_View_Controls::~Tree_View_Controls ( )
{
}


QSize
Tree_View_Controls::minimumSizeHint ( ) const
{
	QSize res;
	int & ww ( res.rwidth() );
	int & hh ( res.rheight() );

	ensurePolished();

	QSize vsbh ( verticalScrollBar()->minimumSizeHint() );

	// Width
	{
		ww = fontMetrics().averageCharWidth() * _min_str_len;
		ww = qMax ( 16, ww );
		ww += indentation();
		ww += vsbh.width();
	}


	// Height
	{
		int rh0 ( sizeHintForRow ( 0 ) );
		rh0 = qMax ( rh0,
			itemDelegate()->sizeHint ( viewOptions(), QModelIndex() ).height() );
		hh = rh0 * _show_rows_min;
		hh = qMax ( vsbh.height(), hh );
	}

	const int fw2 (  2*frameWidth() );
	ww += fw2;
	hh += fw2;

	return res;
}


QSize
Tree_View_Controls::sizeHint ( ) const
{
	QSize res;
	int & ww ( res.rwidth() );
	int & hh ( res.rheight() );

	ensurePolished();

	QSize vsbh ( verticalScrollBar()->sizeHint() );

	// Width
	{
		ww = fontMetrics().averageCharWidth() * _min_str_len;
		ww += indentation();
		ww += vsbh.width();
	}

	// Height
	{

		int rh0 ( sizeHintForRow ( 0 ) );
		rh0 = qMax ( rh0,
			itemDelegate()->sizeHint ( viewOptions(), QModelIndex() ).height() );
		hh = rh0 * _show_rows_avrg;
		hh = qMax ( vsbh.height(), hh );
	}


	const int fw2 ( 2*frameWidth() );
	ww += fw2;
	hh += fw2;

	return res;
}


void
Tree_View_Controls::setModel (
	QAbstractItemModel * model_n )
{
	if ( model() != 0 ) {
		disconnect ( model(), 0, this, 0 );
		disconnect ( this, 0, model(), 0 );
	}

	QTreeView::setModel ( model_n );

	if ( model() != 0 ) {
		connect ( model(), SIGNAL ( layoutChanged() ),
			this, SLOT ( model_layout_changed() ) );
	}

	model_layout_changed();
}


int
Tree_View_Controls::sizeHintForColumn (
	int column ) const
{
	int ww ( 16 );
	ww = qMax ( fontMetrics().averageCharWidth() * _min_str_len, ww );
	ww = qMax ( QTreeView::sizeHintForColumn ( column ), ww );
	return ww;
}


void
Tree_View_Controls::model_layout_changed ( )
{
	update_maximum_size();
	updateGeometry();
	emit sig_layout_changed();
}


void
Tree_View_Controls::update_maximum_size ( )
{
	int max_len ( 16 );

	if ( model() != 0 ) {
		max_len = qMax ( sizeHintForColumn ( 0 ), max_len );
	}

	bool vscroll ( verticalScrollBarPolicy() != Qt::ScrollBarAlwaysOff );
	if ( vscroll ) {
		max_len += verticalScrollBar()->sizeHint().width();
	}
	max_len += 2*frameWidth();

	max_len = qMax ( sizeHint().width(), max_len );

	setMaximumWidth ( max_len );
}


void
Tree_View_Controls::currentChanged (
	const QModelIndex & current,
	const QModelIndex & previous )
{
	QTreeView::currentChanged ( current, previous );
	emit activated ( current );
}


} // End of namespace

